from hrflow_connectors.core.connector import (  # noqa
    BaseActionParameters,
    Connector,
    ConnectorAction,
    Event,
    WorkflowType,
)
from hrflow_connectors.core.warehouse import (  # noqa
    ActionEndpoints,
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
    WarehouseWriteAction,
)



import typing as t

from hrflow_connectors.connectors.hrflow.warehouse.profile import HrFlowProfileWarehouse
from hrflow_connectors.connectors.hubspot.warehouse import HubspotContactWarehouse
from hrflow_connectors.core import BaseActionParameters, Connector, ConnectorAction
from hrflow_connectors.core.connector import WorkflowType


def format_profile(hrflow_profile: t.Dict) -> t.Dict:
    hrflow_profile_info = hrflow_profile["info"]
    hrflow_location = hrflow_profile_info["location"]
    fields = hrflow_location["fields"] or {}
    properties = dict(
        email=hrflow_profile_info["email"],
        firstname=hrflow_profile_info["first_name"],
        lastname=hrflow_profile_info["last_name"],
        date_of_birth=hrflow_profile_info["date_birth"],
        phone=hrflow_profile_info["phone"],
        address=hrflow_location["text"],
        zip=fields.get("postcode") or "Undefined",
        city=fields.get("city") or "Undefined",
        state=fields.get("state") or "Undefined",
        country=fields.get("country") or "Undefined",
        company="",
    )
    profile = dict(properties=properties)
    return profile


Hubspot = Connector(
    name="Hubspot",
    description="",
    url="https://www.hubspot.com/",
    actions=[
        ConnectorAction(
            name="push_profile",
            trigger_type=WorkflowType.catch,
            description=(
                "Writes a profile from Hrflow.ai Source as a contact on Hubspot via"
                " the API"
            ),
            parameters=BaseActionParameters.with_defaults(
                "WriteProfileActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=HubspotContactWarehouse,
        ),
    ],
)



import typing as t

from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.smartrecruiters.warehouse import (
    SmartRecruitersJobWarehouse,
    SmartRecruitersProfileWarehouse,
)
from hrflow_connectors.core import (
    BaseActionParameters,
    Connector,
    ConnectorAction,
    WorkflowType,
)


def get_job_location(smartrecruiters_location: t.Union[t.Dict, None]) -> t.Dict:
    if smartrecruiters_location is None:
        return dict(lat=None, lng=None, text="")

    lat = smartrecruiters_location.get("latitude")
    lat = float(lat) if lat is not None else lat

    lng = smartrecruiters_location.get("longitude")
    lng = float(lng) if lng is not None else lng

    concatenate = []
    for field in ["country", "region", "city", "address"]:
        if smartrecruiters_location.get(field):
            concatenate.append(smartrecruiters_location.get(field))

    return dict(lat=lat, lng=lng, text=" ".join(concatenate))


def get_sections(smartrecruiters_job: t.Dict) -> t.List[t.Dict]:
    sections = []
    if (
        "jobAd" not in smartrecruiters_job
        or "sections" not in smartrecruiters_job["jobAd"]
    ):
        return sections

    smartrecruiters_sections = smartrecruiters_job["jobAd"]["sections"]
    for section_name in [
        "companyDescription",
        "jobDescription",
        "qualifications",
        "additionalInformation",
    ]:
        section = smartrecruiters_sections.get(section_name)
        if section is not None:
            sections.append(
                dict(
                    name="smartrecruiters_jobAd-sections-{}".format(section_name),
                    title=section.get("title"),
                    description=section.get("text"),
                )
            )


def get_tags(smartrecruiters_job: t.Dict) -> t.List[t.Dict]:
    job = smartrecruiters_job
    creator = job.get("creator", {})
    compensation = job.get("compensation", {})

    t = lambda name, value: dict(name=name, value=value)
    return [
        t("smartrecruiters_status", job.get("status")),
        t("smartrecruiters_postingStatus", job.get("postingStatus")),
        t("smartrecruiters_id", job.get("id")),
        t(
            "smartrecruiters_experienceLevel-id",
            job.get("experienceLevel", {}).get("id"),
        ),
        t(
            "smartrecruiters_typeOfEmployment-id",
            job.get("typeOfEmployment", {}).get("id"),
        ),
        t("smartrecruiters_compensation-min", compensation.get("min")),
        t("smartrecruiters_compensation-max", compensation.get("max")),
        t("smartrecruiters_compensation-currency", compensation.get("currency")),
        t("smartrecruiters_industry-id", job.get("industry", {}).get("id")),
        t("smartrecruiters_creator-firstName", creator.get("firstName")),
        t("smartrecruiters_creator-lastName", creator.get("lastName")),
        t("smartrecruiters_function-id", job.get("function", {}).get("id")),
        t("smartrecruiters_department-id", job.get("department", {}).get("id")),
        t("smartrecruiters_location-manual", job.get("location", {}).get("manual")),
        t("smartrecruiters_location-remote", job.get("location", {}).get("remote")),
        t("smartrecruiters_eeoCategory-id", job.get("eeoCategory", {}).get("id")),
        t("smartrecruiters_targetHiringDate", job.get("targetHiringDate")),
    ]


def format_job(smartrecruiters_job: t.Dict) -> t.Dict:
    job = dict(
        name=smartrecruiters_job.get("title", "Undefined"),
        reference=smartrecruiters_job.get("refNumber"),
        created_at=smartrecruiters_job.get("createdon"),
        updated_at=smartrecruiters_job.get("updatedon"),
        location=get_job_location(smartrecruiters_job.get("location")),
        url=None,
        summary=None,
        sections=get_sections(smartrecruiters_job),
        tags=get_tags(smartrecruiters_job),
    )
    return job


def get_profile_location(hrflow_location: t.Dict) -> t.Dict:
    fields = hrflow_location["fields"] or {}
    return dict(
        lat=hrflow_location["lat"] or 0,
        lng=hrflow_location["lng"] or 0,
        city=fields.get("city") or "Undefined",
        country=fields.get("country") or "Undefined",
        region=fields.get("state") or "Undefined",
    )


def get_profile_occupation(hrflow_occupation: t.Dict) -> t.Dict:
    return dict(
        description=hrflow_occupation["description"],
        # FIXME Maybe that current could be True for the most recent occupation
        current=False,
        startDate=(hrflow_occupation["date_start"] or "XXXX").split("T")[0],
        endDate=(hrflow_occupation["date_end"] or "XXXX").split("T")[0],
        location=hrflow_occupation["location"]["text"] or "Undefined",
    )


def get_profile_experiences(hrflow_experiences: t.List[t.Dict]) -> t.List[t.Dict]:
    return [
        dict(
            title=experience["title"] or "Undefined",
            company=experience["company"] or "Undefined",
            **get_profile_occupation(experience)
        )
        for experience in hrflow_experiences
    ]


def get_profile_educations(hrflow_educations: t.List[t.Dict]) -> t.List[t.Dict]:
    return [
        dict(
            institution=education["school"] or "Undefined",
            degree=education["title"] or "Undefined",
            major="Undefined",
            **get_profile_occupation(education)
        )
        for education in hrflow_educations
    ]


def format_profile(hrflow_profile: t.Dict) -> t.Dict:
    hrflow_profile_info = hrflow_profile["info"]

    profile = dict(
        firstName=hrflow_profile_info["first_name"],
        lastName=hrflow_profile_info["last_name"],
        email=hrflow_profile_info["email"],
        phoneNumber=hrflow_profile_info["phone"],
        location=get_profile_location(hrflow_profile_info["location"]),
        experiences=get_profile_experiences(hrflow_profile["experiences"]),
        educations=get_profile_educations(hrflow_profile["educations"]),
        web=dict(hrflow_profile_info["urls"]),
        tags=[],
        consent=True,
        attachments=hrflow_profile.get("attachments") or [],
    )
    return profile


DESCRIPTION = (
    "Move beyond applicant tracking systems (ATS) with an enterprise-grade recruiting"
    " platform designed for the modern workforce. SmartRecruiters' Talent Acquisition"
    " Suite provides everything needed to attract, select, and hire great talent."
)
SmartRecruiters = Connector(
    name="SmartRecruiters",
    description=DESCRIPTION,
    url="https://www.smartrecruiters.com/",
    actions=[
        ConnectorAction(
            name="pull_jobs",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs via the ***SmartRecruiter*** API and send them"
                " to a ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadJobsActionParameters", format=format_job
            ),
            origin=SmartRecruitersJobWarehouse,
            target=HrFlowJobWarehouse,
        ),
        ConnectorAction(
            name="push_profile",
            trigger_type=WorkflowType.catch,
            description=(
                "Writes a profile from Hrflow.ai Source to SmartRecruiters via the API"
                " for the given `job_id`."
            ),
            parameters=BaseActionParameters.with_defaults(
                "WriteProfileActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=SmartRecruitersProfileWarehouse,
        ),
    ],
)



import enum
import typing as t
from logging import LoggerAdapter

import requests
from pydantic import Field

from hrflow_connectors.connectors.smartrecruiters.schemas import (
    SmartRecruitersJob,
    SmartRecruitersProfile,
)
from hrflow_connectors.core import (
    ActionEndpoints,
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
    WarehouseWriteAction,
)

SMARTRECRUITERS_JOBS_ENDPOINT = "https://api.smartrecruiters.com/jobs"
SMARTRECRUITERS_JOBS_ENDPOINT_LIMIT = 100

GET_ALL_JOBS_ENDPOINT = ActionEndpoints(
    name="Get all jobs",
    description=(
        "Endpoint to search jobs by traditional params (offset, limit...)"
        " and get the list of all jobs with their ids, the request method"
        " is `GET`"
    ),
    url=(
        "https://dev.smartrecruiters.com/customer-api/live-docs/job-api/#/jobs/jobs.all"
    ),
)
GET_JOB_ENDPOINT = ActionEndpoints(
    name="Get job",
    description=(
        "Endpoint to get the content of a job with a given id, a jobId parameter is"
        " required, the request method is `GET`"
    ),
    url=(
        "https://dev.smartrecruiters.com/customer-api/live-docs/job-api/#/jobs/jobs.get"
    ),
)
POST_CANDIDATE_ENDPOINT = ActionEndpoints(
    name="Post Candidate",
    description=(
        "Endpoint to create a new candidate and assign to a talent pool, the request"
        " method is `POST`"
    ),
    url="https://dev.smartrecruiters.com/customer-api/live-docs/candidate-api/",
)


class JobPostingStatus(str, enum.Enum):
    public = "PUBLIC"
    internal = "INTERNAL"
    not_published = "NOT_PUBLISHED"
    private = "PRIVATE"


class JobStatus(str, enum.Enum):
    created = "CREATED"
    sourcing = "SOURCING"
    filled = "FILLED"
    interview = "INTERVIEW"
    offer = "OFFER"
    cancelled = "CANCELLED"
    on_hold = "ON_HOLD"


class WriteProfilesParameters(ParametersModel):
    x_smart_token: str = Field(
        ...,
        description="X-SmartToken used to access SmartRecruiters API",
        repr=False,
        field_type=FieldType.Auth,
    )
    job_id: str = Field(
        ...,
        description=(
            "Id of a Job to which you want to assign a candidates "
            "when it’s created. Profiles are sent to this "
            "URL `https://api.smartrecruiters.com/jobs/{job_id}/candidates` "
        ),
        field_type=FieldType.QueryParam,
    )


class ReadJobsParameters(ParametersModel):
    x_smart_token: str = Field(
        ...,
        description="X-SmartToken used to access SmartRecruiters API",
        repr=False,
        field_type=FieldType.Auth,
    )
    query: t.Optional[str] = Field(
        None,
        description=(
            "Case insensitive full-text query against job title e.g. java developer"
        ),
        field_type=FieldType.QueryParam,
    )
    updated_after: t.Optional[str] = Field(
        None,
        description="ISO8601-formatted time boundaries for the job update time",
        field_type=FieldType.QueryParam,
    )
    posting_status: t.Optional[JobPostingStatus] = Field(
        None,
        description="Posting status of a job. One of {}".format(
            [e.value for e in JobPostingStatus]
        ),
        field_type=FieldType.QueryParam,
    )
    job_status: t.Optional[JobStatus] = Field(
        None,
        description="Status of a job. One of {}".format([e.value for e in JobStatus]),
        field_type=FieldType.QueryParam,
    )
    limit: t.Optional[int] = Field(
        SMARTRECRUITERS_JOBS_ENDPOINT_LIMIT,
        description=(
            "Number of items to pull from SmartRecruiters at a time. Not matter what"
            " value is supplied it is capped at {}".format(
                SMARTRECRUITERS_JOBS_ENDPOINT_LIMIT
            )
        ),
        field_type=FieldType.QueryParam,
    )


def read(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    page = None
    while True:
        params = dict(
            q=parameters.query,
            updatedAfter=parameters.updated_after,
            postingStatus=parameters.posting_status,
            status=parameters.job_status,
            limit=min(parameters.limit, SMARTRECRUITERS_JOBS_ENDPOINT_LIMIT),
            pageId=page,
        )
        response = requests.get(
            SMARTRECRUITERS_JOBS_ENDPOINT,
            headers={"X-SmartToken": parameters.x_smart_token},
            params=params,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to pull jobs from SmartRecruiters params={}"
                " status_code={} response={}".format(
                    params, response.status_code, response.text
                )
            )
            raise Exception("Failed to pull jobs from SmartRecruiters")
        response = response.json()
        jobs = response["content"]
        if len(jobs) == 0:
            break

        adapter.info(
            "Pulling {} jobs from page {} out of total jobs {}".format(
                len(jobs), page if page is not None else 1, response["totalFound"]
            )
        )
        for job in jobs:
            full_job_response = requests.get(
                "{}/{}".format(SMARTRECRUITERS_JOBS_ENDPOINT, job["id"]),
                headers={"X-SmartToken": parameters.x_smart_token},
            )
            if full_job_response.status_code // 100 != 2:
                adapter.error(
                    "Failed to pull jobs details from SmartRecruiters job_id={}"
                    " status_code={} response={}".format(
                        job["id"],
                        full_job_response.status_code,
                        full_job_response.text,
                    )
                )
                raise Exception("Failed to pull jobs from SmartRecruiters")
            yield full_job_response.json()

        page = response["nextPageId"]
        if not page:
            break


def write(
    adapter: LoggerAdapter,
    parameters: WriteProfilesParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    adapter.info(
        "Pushing {} profiles with job_id={}".format(len(profiles), parameters.job_id)
    )
    failed_profiles = []
    for profile in profiles:
        response = requests.post(
            "{}/{}/candidates".format(SMARTRECRUITERS_JOBS_ENDPOINT, parameters.job_id),
            headers={"X-SmartToken": parameters.x_smart_token},
            json=profile,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to push profile to SmartRecruiters job_id={}"
                " status_code={} response={}".format(
                    parameters.job_id,
                    response.status_code,
                    response.text,
                )
            )
            failed_profiles.append(profile)
    return failed_profiles


SmartRecruitersJobWarehouse = Warehouse(
    name="SmartRecruiters Jobs",
    data_schema=SmartRecruitersJob,
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=ReadJobsParameters,
        function=read,
        endpoints=[GET_ALL_JOBS_ENDPOINT, GET_JOB_ENDPOINT],
    ),
)

SmartRecruitersProfileWarehouse = Warehouse(
    name="SmartRecruiters Profiles",
    data_schema=SmartRecruitersProfile,
    data_type=DataType.profile,
    write=WarehouseWriteAction(
        parameters=WriteProfilesParameters,
        function=write,
        endpoints=[POST_CANDIDATE_ENDPOINT],
    ),
)



from typing import Any, List, Optional

from pydantic import BaseModel, Field


# Job Model
class Location(BaseModel):
    name: str


class Department(BaseModel):
    id: int
    name: str
    parent_id: Any
    child_ids: Optional[List[int]]
    external_id: int


class Office(BaseModel):
    id: int
    name: str
    location: str
    parent_id: Optional[int]
    child_ids: Optional[List[int]]


class Salary_range(BaseModel):
    min_value: int
    max_value: int
    unit: str


class Custom_fields(BaseModel):
    employment_type: str
    maximum_budget: str
    salary_range: Salary_range


class Employement_type(BaseModel):
    name: str
    type: str
    value: str


class Budget(BaseModel):
    name: str
    type: str
    value: str


class Keyed_custom_fields(BaseModel):
    employement_type = Employement_type
    budget: Budget
    salary_range: Salary_range


class Team_model(BaseModel):
    id: int
    first_name: str
    last_name: str
    name: str
    employee_id: str
    responsible: Optional[bool]


class Hiring_team(BaseModel):
    hiring_managers: List[Team_model]
    recruiters: List[Team_model]
    coordinators: List[Team_model]
    sourcers: List[Team_model]


class Close_reason(BaseModel):
    id: int
    name: str


class Opening(BaseModel):
    id: int
    opening_id: str
    status: str
    opened_at: str
    closed_at: str
    application_id: int
    close_reason: Close_reason


class GreenhouseJobModel(BaseModel):
    id: int
    internal_job_id: int
    title: str
    updated_at: Optional[str]
    requisition_id: Optional[str]
    location: Location
    absolute_url: str
    metadata: Any
    content: str
    departments: List[Department]
    offices: List[Office]


# Profile Model
class PhoneNumber(BaseModel):
    value: str
    type: str


class Address(BaseModel):
    value: str
    type: str


class EmailAddress(BaseModel):
    value: str
    type: str


class WebsiteAddress(BaseModel):
    value: str
    type: str


class SocialMediaAddress(BaseModel):
    value: str


class Education(BaseModel):
    school_id: int
    discipline_id: int
    degree_id: int
    start_date: str
    end_date: str


class Employment(BaseModel):
    company_name: str
    title: str
    start_date: str
    end_date: str


class Recruiter(BaseModel):
    id: Optional[int]
    email: Optional[str]


class coordinator(BaseModel):
    id: int
    email: str


class GreenhouseProfileModel(BaseModel):
    first_name: str = Field(..., description="The candidate's first name")
    last_name: str = Field(..., description="The candidate's last name")
    company: Optional[str] = Field(None, description="The candidate's company'")
    title: Optional[str] = Field(None, description="The candidate's title'")
    phone_numbers: Optional[List[PhoneNumber]] = Field(
        None,
        description="Array of phone numbers. Passing an empty array will clear all",
    )
    addresses: Optional[List[Address]] = Field(
        None, description="Array of addresses, passing an empty array will clear all"
    )
    email_addresses: Optional[List[EmailAddress]] = Field(
        None, description="Array of email addresses, passing an empty array will"
    )
    website_addresses: Optional[List[WebsiteAddress]] = Field(
        None,
        description="Array of website addresses, passing an empty array will clear all",
    )
    social_media_addresses: Optional[List[SocialMediaAddress]] = Field(
        None,
        description=(
            "Array of social media addresses. Passing an empty array will clear all"
        ),
    )
    educations: Optional[List[Education]] = Field(
        None, description="Array of education records"
    )
    employments: Optional[List[Employment]] = Field(
        None, description="Array of employment records"
    )
    tags: Optional[List[str]] = Field(
        None,
        description="Array of tags as strings. Passing an empty array will clear all",
    )
    applications: List[int] = Field(
        ...,
        description=(
            "An array of application objects `dict(job_id=int)`. at least one is"
            " required"
        ),
    )
    recruiter: Optional[Recruiter] = Field(
        None, description="An object representing the candidate's recruiter"
    )
    coordinator: Optional[coordinator]



import typing as t
from datetime import datetime

import requests
from pydantic import BaseModel

from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileParsingWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.talentsoft.warehouse import (
    TalentSoftJobsWarehouse,
    TalentSoftProfilesWarehouse,
    get_talentsoft_auth_token,
)
from hrflow_connectors.core import (
    BaseActionParameters,
    Connector,
    ConnectorAction,
    Event,
    WorkflowType,
)


def format_ts_vacancy(ts_vacancy: t.Dict) -> t.Dict:
    # FIXME lat and lng makes requests to HERE Maps API in original workflow
    ts_location = ts_vacancy["location"]
    location = dict(
        text=ts_location["address"],
        lat=ts_location["latitude"],
        lng=ts_location["longitude"],
    )

    job_description = ts_vacancy["jobDescription"]
    custom_fields = job_description["jobDescriptionCustomFields"]
    sections = [
        dict(
            name="description1",
            title="description1",
            description=job_description["description1"] or "",
        ),
        dict(
            name="description2",
            title="description2",
            description=job_description["description2"] or "",
        ),
        dict(
            name="Complément du descriptif",
            title="Complément du descriptif",
            description="\n".join(
                [
                    custom_fields.get("longText1") or "",
                    custom_fields.get("longText2") or "",
                    custom_fields.get("longText3") or "",
                ]
            ),
        ),
    ]

    languages = [
        dict(name=language["language"]["label"], value=None)
        for language in ts_vacancy.get("languages") or []
    ]

    tags = [
        dict(
            name="talentsoft-organisation-id",
            value=(ts_vacancy["organisation"] or {}).get("id"),
        ),
        dict(
            name="talentsoft-status-id",
            value=(ts_vacancy["status"] or {}).get("id"),
        ),
        dict(
            name="talentsoft-professionalCategory-id",
            value=(ts_vacancy["jobDescription"]["professionalCategory"] or {}).get(
                "id"
            ),
        ),
        dict(
            name="talentsoft-country-id",
            value=(ts_vacancy["jobDescription"]["country"] or {}).get("id"),
        ),
        dict(
            name="talentsoft-primaryProfile-id",
            value=(ts_vacancy["jobDescription"]["primaryProfile"] or {}).get("id"),
        ),
        dict(
            name="talentsoft-contractType-id",
            value=(ts_vacancy["jobDescription"]["contractType"] or {}).get("id"),
        ),
        dict(
            name="talentsoft-publishedOnInternet",
            value=ts_vacancy.get("publishedOnInternet"),
        ),
        dict(
            name="talentsoft-publishedOnIntranet",
            value=ts_vacancy.get("publishedOnIntranet"),
        ),
    ]

    if ts_vacancy["criteria"]["experienceLevel"]:
        tags.append(
            dict(
                name="talentsoft-experienceLevel",
                value=ts_vacancy["criteria"]["experienceLevel"].get("id"),
            )
        )

    if ts_vacancy["criteria"]["educationLevel"]:
        tags.append(
            dict(
                name="talentsoft-educationLevel",
                value=ts_vacancy["criteria"]["educationLevel"].get("id"),
            )
        )

    return dict(
        name=ts_vacancy["jobDescription"]["title"],
        reference=ts_vacancy["reference"],
        created_at=ts_vacancy["creationDate"],
        location=location,
        url=None,
        summary=None,
        sections=sections,
        tags=tags,
        skills=ts_vacancy["criteria"]["skills"],
        languages=languages,
    )


def format_ts_candidate(ts_candidate: t.Dict) -> t.Dict:
    details = ts_candidate["candidateDetail"]
    tags = [
        dict(name="talentsoft-isEmployee", value=ts_candidate["isEmployee"]),
        dict(name="talentsoft-isInProgress", value=ts_candidate["isInProgress"]),
        dict(
            name="talentsoft-residentCountry-id",
            value=details["personalInformation"]["residentCountry"].get("id"),
        ),
    ]

    contract_type = details["positionSought"]["contractType"]
    if contract_type:
        tags.append(
            dict(
                name="talentsoft-contractType-id",
                value=contract_type["id"],
            )
        )

    custom_code_table_1 = details["positionSought"]["jobPreferencesCustomFields"][
        "customCodeTable1"
    ]
    if custom_code_table_1:
        tags.append(
            dict(
                name="talentsoft-profileStatus-id",
                value=custom_code_table_1["id"],
            )
        )

    global_experience_level = details["globalExperience"]["globalExperienceLevel"]
    if global_experience_level:
        tags.append(
            dict(
                name="talentsoft-experienceLevel-id",
                value=global_experience_level["id"],
            )
        )

    primary_profile = details["positionSought"]["primaryProfile"]
    if primary_profile:
        tags.append(
            dict(
                name="talentsoft-profile-id",
                value=primary_profile["id"],
            )
        )

    for education in details["educations"]:
        if education["educationLevel"]:
            tags.append(
                {
                    "name": "talentsofteducationLevel-id",
                    "value": education["educationLevel"]["id"],
                }
            )
    for application in ts_candidate["applications"]:
        tags.append(
            dict(
                name="talentsoft-application-vacancyReference",
                value=application["vacancyReference"],
            )
        )

    metadatas = [
        dict(name="profile_uid", value=details["id"]),
    ]

    resume = None
    attachment = next(
        (
            attachment
            for attachment in ts_candidate["attachments"]
            if attachment["isResume"]
        ),
        None,
    )
    if attachment:
        resume = dict(raw=attachment["raw"], content_type=attachment["mimeType"])
        metadatas.append(dict(name="filename", value=attachment["filename"]))
    return dict(
        reference=details["id"],
        created_at=details["creationDate"],
        updated_at=datetime.utcnow().isoformat(),
        resume=resume,
        tags=tags,
        metadatas=metadatas,
    )


def ts_callback(
    origin_parameters: BaseModel,
    target_parameters: BaseModel,
    events: t.Counter[Event],
    items_to_write: t.List[t.Dict],
) -> None:
    if len(items_to_write) == 1 and events[Event.write_failure] == 0:
        candidate = items_to_write[0]
        token = get_talentsoft_auth_token(
            client_url=origin_parameters.client_url,
            client_id=origin_parameters.client_id,
            client_secret=origin_parameters.client_secret,
        )
        report = dict(
            reportType="Candidates",
            items=[
                dict(
                    id=candidate["reference"],
                    date=candidate["created_at"],
                    lastUpdateDate=candidate["updated_at"],
                    status="success",
                )
            ],
        )
        response = requests.post(
            "{}/api/exports/v1/reports".format(origin_parameters.client_url),
            headers={
                "Authorization": "bearer {}".format(token),
            },
            json=report,
        )
        if not response.ok:
            raise Exception(
                "Talentsoft callback report failed error={} evens={}".format(
                    response.text,
                    events,
                )
            )


def applicant_new_parser(event: t.Dict) -> t.Dict:
    return dict(filter="id::{}".format(event["applicantId"]))


def applicant_resume_update_parser(event: t.Dict) -> t.Dict:
    return dict(filter="id::{}".format(event["applicantId"]), fileId=event["fileId"])


def applicant_update_parser(event: t.Dict) -> t.Dict:
    return dict(filter="id::{}".format(event["applicantId"]))


DESCRIPTION = "TalentSoft"
TalentSoft = Connector(
    name="TalentSoft",
    description=DESCRIPTION,
    url="https://www.cegid.com/fr/produits/cegid-talentsoft/",
    actions=[
        ConnectorAction(
            name="applicant_new",
            trigger_type=WorkflowType.catch,
            description=(
                "Handle TalentSoft 'applicant_new' event by fetching profile from"
                " TalentSoft and sending it to HrFlow.ai Parsing API."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ApplicantNewActionParameters",
                format=format_ts_candidate,
                event_parser=applicant_new_parser,
            ),
            origin=TalentSoftProfilesWarehouse.with_fixed_read_parameters(
                only_resume=True
            ),
            target=HrFlowProfileParsingWarehouse.with_fixed_write_parameters(
                only_insert=True
            ),
            callback=ts_callback,
        ),
        ConnectorAction(
            name="applicant_resume_update",
            trigger_type=WorkflowType.catch,
            description=(
                "Handle TalentSoft 'applicant_resume_update' event by"
                " running a new HrFlow.ai Parsing on updated resume."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ApplicantResumeUpdateActionParameters",
                format=format_ts_candidate,
                event_parser=applicant_resume_update_parser,
            ),
            origin=TalentSoftProfilesWarehouse.with_fixed_read_parameters(
                only_resume=True
            ),
            target=HrFlowProfileParsingWarehouse.with_fixed_write_parameters(
                only_insert=False
            ),
            callback=ts_callback,
        ),
        ConnectorAction(
            name="applicant_update",
            trigger_type=WorkflowType.catch,
            description=(
                "Handle TalentSoft 'applicant_update' event by"
                " only updating tags coming from TalentSoft in HrFlow.ai."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ApplicantUpdateActionParameters",
                format=format_ts_candidate,
                event_parser=applicant_update_parser,
            ),
            origin=TalentSoftProfilesWarehouse,
            target=HrFlowProfileWarehouse.with_fixed_write_parameters(
                edit=True, only_edit_fields=["tags"]
            ),
        ),
        ConnectorAction(
            name="pull_profiles",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves profiles from TalentSoft candidates export API and send them"
                " to a ***Hrflow.ai Source***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "PullTalentSoftProfilesActionParameters",
                format=format_ts_candidate,
            ),
            origin=TalentSoftProfilesWarehouse.with_fixed_read_parameters(
                only_resume=True
            ),
            target=HrFlowProfileParsingWarehouse,
        ),
        ConnectorAction(
            name="pull_jobs",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves jobs from TalentSoft vacancies export API and send them"
                " to a ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "PullTalentSoftProfilesActionParameters",
                format=format_ts_vacancy,
            ),
            origin=TalentSoftJobsWarehouse,
            target=HrFlowJobWarehouse,
        ),
    ],
)



import typing as t
from logging import LoggerAdapter

import requests
from pydantic import Field

from hrflow_connectors.connectors.sapsuccessfactors.schemas import (
    SapCandidateModel,
    SAPSuccessFactorsJob,
)
from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
    WarehouseWriteAction,
)

SAP_JOBS_ENDPOINT_LIMIT = 100


class WriteProfilesParameters(ParametersModel):
    api_server: str = Field(
        ..., description="Server to be accessed", repr=False, field_type=FieldType.Other
    )
    api_key: str = Field(
        ...,
        description="API Key used to authenticate on the SAP API",
        field_type=FieldType.Auth,
    )


class ReadProfilesParameters(ParametersModel):
    api_server: str = Field(
        ..., description="Server to be accessed", repr=False, field_type=FieldType.Other
    )
    api_key: str = Field(
        ...,
        description="API Key used to authenticate on the SAP API",
        field_type=FieldType.Auth,
    )
    top: t.Optional[int] = Field(
        SAP_JOBS_ENDPOINT_LIMIT,
        description="Show only the first N items value is capped at {}".format(
            SAP_JOBS_ENDPOINT_LIMIT
        ),
        field_type=FieldType.QueryParam,
    )
    skip: t.Optional[int] = Field(
        description="Search items by search phrases", field_type=FieldType.QueryParam
    )
    filter: t.Optional[str] = Field(
        description="Filter items by property values",
        repr=False,
        field_type=FieldType.QueryParam,
    )
    search: t.Optional[str] = Field(
        description="Search items by search phrases",
        repr=False,
        field_type=FieldType.QueryParam,
    )


class ReadJobsParameters(ParametersModel):
    api_server: str = Field(
        ..., description="Server to be accessed", repr=False, field_type=FieldType.Other
    )
    api_key: str = Field(
        ...,
        description="API Key used to authenticate on the SAP API",
        field_type=FieldType.Auth,
    )
    top: t.Optional[int] = Field(
        SAP_JOBS_ENDPOINT_LIMIT,
        description="Show only the first N items value is capped at {}".format(
            SAP_JOBS_ENDPOINT_LIMIT
        ),
        field_type=FieldType.QueryParam,
    )
    skip: t.Optional[int] = Field(
        description="Search items by search phrases", field_type=FieldType.QueryParam
    )
    filter: t.Optional[str] = Field(
        description="Filter items by property values",
        repr=False,
        field_type=FieldType.QueryParam,
    )
    search: t.Optional[str] = Field(
        description="Search items by search phrases",
        repr=False,
        field_type=FieldType.QueryParam,
    )


# enriches a job object with it's requisition field
def enrich_requisition(parameters: ReadJobsParameters, job: t.Dict) -> t.Dict:
    job_and_requisition = dict()
    job_and_requisition["job"] = job

    url = job.get("jobRequisition").get("__deferred").get("uri")
    requisition = requests.get(
        url, headers={"APIKey": parameters.api_key, "Accept": "application/json"}
    )
    requisition = requisition.json()
    job_and_requisition["requisition"] = requisition.get("d")
    return job_and_requisition


def read(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:

    params = dict()
    params["top"] = parameters.top
    params["skip"] = parameters.skip
    params["filter"] = parameters.filter
    params["search"] = parameters.search

    url = f"https://{parameters.api_server}/odata/v2/JobRequisitionLocale?%24top=20"

    response = requests.get(
        url,
        headers={"APIKey": parameters.api_key, "Accept": "application/json"},
        params=params,
    )
    if response.status_code // 100 != 2:
        adapter.error(
            "Failed to pull jobs from SAP, status_code={}".format(response.status_code)
        )
        raise Exception("Failed to pull jobs from SAP")
    response = response.json()
    job_list = response.get("d").get("results")
    job_res = []
    for job in job_list:
        job_res.append(enrich_requisition(parameters, job))
    return job_res


def write(
    adapter: LoggerAdapter,
    parameters: WriteProfilesParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    adapter.info("Pushing {} profiles".format(len(profiles)))
    failed_profiles = []
    for profile in profiles:
        response = requests.post(
            "https://{}/odata/v2/Candidate".format(parameters.api_server),
            headers={"APIKey": parameters.api_key},
            json=profile,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to push profile to SAP status_code={} response={}".format(
                    response.status_code,
                    response.text,
                )
            )
            failed_profiles.append(profile)
    return failed_profiles


def read_parsing(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:

    params = dict()
    params["top"] = parameters.top
    params["skip"] = parameters.skip
    params["filter"] = parameters.filter
    params["search"] = parameters.search

    url = f"{parameters.api_server}/odata/v2/Candidate"
    headers = {
        "api_key": parameters.api_key,
        "Content-Type": "application/json",
        "Accept": "application/pdf",  # Request PDF format for the CV content
    }

    # Set the parameters for the API call
    params = {
        "$select": "ID,Resume",  # Only retrieve the ID and Resume fields
        "$expand": "Resume($select=ID,Content)",  # Also retrieve the ID
        # and Content of the Resume entity
    }

    # Make the GET request
    response = requests.get(url, headers=headers, params=params)

    # Check if the request was successful
    if response.status_code != 200:
        adapter.error(f"Failed to retrieve candidate CVs. Response: {response.text}")
        raise Exception(f"Failed to retrieve candidate CVs. Response: {response.text}")

    # Extract the candidate data from the response
    candidates_data = response.json()["value"]

    # Extract the CV content for each candidate
    cvs = []
    for candidate in candidates_data:
        cv_content = candidate["Resume"]["Content"]
        cvs.append(cv_content)

    return cvs


def read_profiles(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:

    params = dict()
    params["top"] = parameters.top
    params["skip"] = parameters.skip
    params["filter"] = parameters.filter
    params["search"] = parameters.search

    # Set the API endpoint URL
    endpoint_url = f"{parameters.api_server}/odata/v2/Candidate"

    # Set the headers with the API key
    headers = {
        "api_key": parameters.api_key,
        "Content-Type": "application/json",
    }

    # Set the parameters for the API call
    params = {
        "$select": "ID,FirstName,LastName",  # Only retrieve the ID,
        # FirstName, and LastName fields
    }

    # Make the GET request
    response = requests.get(endpoint_url, headers=headers, params=params)

    # Check if the request was successful
    if response.status_code != 200:
        adapter.error(f"Failed to retrieve candidates. Response: {response.text}")
        raise Exception(f"Failed to retrieve candidates. Response: {response.text}")

    # Extract the candidate data from the response
    candidates_data = response.json()["value"]

    return candidates_data


SAPProfileWarehouse = Warehouse(
    name="SAP Profiles",
    data_schema=SapCandidateModel,
    data_type=DataType.profile,
    write=WarehouseWriteAction(
        parameters=WriteProfilesParameters,
        function=write,
        endpoints=[],
    ),
)

SAPProfileParsingWarehouse = Warehouse(
    name="SAP Profiles Parsing",
    data_schema=SapCandidateModel,
    data_type=DataType.profile,
    read=WarehouseReadAction(
        parameters=ReadProfilesParameters,
        function=read_parsing,
        endpoints=[],
    ),
)

SAPJobWarehouse = Warehouse(
    name="SAP Job",
    data_schema=SAPSuccessFactorsJob,
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=ReadJobsParameters,
        function=read,
        endpoints=[],
    ),
)



import datetime
import re
import typing as t
from typing import Any, Dict

from hrflow_connectors.connectors.hrflow.schemas import HrFlowProfile
from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.sapsuccessfactors.warehouse import (
    SAPJobWarehouse,
    SAPProfileWarehouse,
)
from hrflow_connectors.core import (
    BaseActionParameters,
    Connector,
    ConnectorAction,
    WorkflowType,
)

from .schemas import SAPSuccessFactorsJob
from .utils.datetime_converter import from_str_to_datetime

DEFAULT_COUNTRY = "France"
DEFAULT_JOB_NAME = "Undefined"
DEFAULT_ADDRESS = "Undefined"


def remove_html_tags(text: str) -> str:
    """
    Remove all HTML tags in a string
    Args:
        text (str): text to clean
    Returns:
        str: cleaned text (without HTML tags)
    """
    return re.sub("<[^<]+?>", "", text)


def format_date(date: datetime.datetime) -> str:
    """
    Format `datetime.datetime` to SAP Date
    Args:
        date (datetime.datetime): Datetime to format
    Returns:
        str: formatted date
    """
    timestamp = int(date.timestamp())
    sap_date = f"/Date({timestamp})/"
    return sap_date


def format_start_date(date_str: str) -> str:
    """
    Format ISO start date to SAP Date
    Args:
        date_str (str): iso date to format
    Returns:
        str: formatted SAP date
    """
    converted_date = from_str_to_datetime(date_str)
    return format_date(converted_date)


def format_end_date(date_str: str) -> str:
    """
    Format ISO end date to SAP Date
    Args:
        date_str (str): iso date to format
    Returns:
        str: formatted SAP date
    """
    converted_date = from_str_to_datetime(date_str)
    # add 10 seconds to avoid that end_date is less or equal than start_date
    converted_date += datetime.timedelta(seconds=10)
    return format_date(converted_date)


def format_job(data: SAPSuccessFactorsJob) -> t.Dict:
    """
    Format retrieved jobs into a HrFlow job object
    Args:
        data (SAPSuccessFactorsJob): job to format
    Returns:
        HrflowJob: job in the HrFlow job object format
    """

    job = dict()

    job_requisition = data.get("requisition")
    if job_requisition is None:
        job_requisition = dict()
    data = data.get("job")

    # name
    if data.get("jobTitle") is not None:
        job["name"] = data.get("jobTitle")
    else:
        job["name"] = DEFAULT_JOB_NAME

    # reference
    job["reference"] = data.get("jobReqId")

    # location
    geojson = dict(
        city=job_requisition.get("city"),
        country=job_requisition.get("country"),
        facility=job_requisition.get("facility"),
        province=job_requisition.get("stateProvince"),
    )
    job["location"] = dict(
        text=job_requisition.get("location"),
        city=job_requisition.get("city"),
        geojson=geojson,
        lat=None,
        lng=None,
    )

    # sections
    job["sections"] = []
    if data.get("jobDescription") is not None:
        description_text = data.get("jobDescription")
        description_text = remove_html_tags(description_text)
        description_text = description_text.replace("#13;", " ")
        description_text = description_text.replace("&", "")
        description_text = description_text.replace("&nbsp;", "")
        description_text = description_text.replace("quo;s", "")

        description_section = dict(
            name="sap_description",
            title="sap_description",
            description=description_text,
        )
        job["sections"].append(description_section)

    # tags
    t = lambda name, value: dict(name=name, value=value)
    job["tags"] = [
        t("sapsuccessfactors_annual_SA", job_requisition.get("annual_SA")),
        t("sapsuccessfactors_department", job_requisition.get("department")),
        t("sapsuccessfactors_function", job_requisition.get("function")),
        t("sapsuccessfactors_division", job_requisition.get("division")),
        t("sapsuccessfactors_industry", job_requisition.get("industry")),
        t("sapsuccessfactors_monthly_salary", job_requisition.get("monthly_salary")),
        t("sapsuccessfactors_otherBonus", job_requisition.get("otherBonus")),
        t("sapsuccessfactors_salaryBase", job_requisition.get("salaryBase")),
        t("sapsuccessfactors_salaryMax", job_requisition.get("salaryMax")),
        t("sapsuccessfactors_salaryMin", job_requisition.get("salaryMin")),
        t("sapsuccessfactors_jobStartDate", job_requisition.get("jobStartDate")),
    ]

    job["metadatas"] = [
        t("sapsuccessfactors_recruiterTeam", job_requisition.get("recruiterTeam")),
        t("sapsuccessfactors_sourcerTeam", job_requisition.get("sourcerTeam")),
        t(
            "sapsuccessfactors_hiringManagerTeam",
            job_requisition.get("hiringManagerTeam"),
        ),
    ]

    return job


def format_education(education: Dict[str, Any]) -> Dict[str, Any]:
    """
    Format Hrflow education to SAP Education
    Args:
        education (Dict[str, Any]): Hrflow education
    Returns:
        Dict[str, Any]: SAP education
    """
    sap_education = dict()
    date_start = education.get("date_start")
    date_end = education.get("date_end")

    if date_start is not None:
        sap_education["startDate"] = format_start_date(date_start)

    if date_end is not None:
        sap_education["endDate"] = format_end_date(date_end)

    sap_education["school"] = education.get("school")
    school_location = education.get("location", {})
    school_address = school_location.get("text")
    if school_address is None:
        school_address = DEFAULT_ADDRESS
    sap_education["schoolAddress"] = school_address
    return sap_education


def format_experience(experience: Dict[str, Any]) -> Dict[str, Any]:
    """
    Format Hrflow experience to SAP experience
    Args:
        experience (Dict[str, Any]): Hrflow experience
    Returns:
        Dict[str, Any]:SAP experience
    """
    sap_experience = dict()
    sap_experience["employer"] = experience.get("company")

    experience_location = experience.get("location", {})
    experience_address = experience_location.get("text")
    if experience_address is None:
        experience_address = DEFAULT_ADDRESS
    sap_experience["employerAddress"] = experience_address

    date_start = experience.get("date_start")
    date_end = experience.get("date_end")
    if date_start is not None:
        sap_experience["startDate"] = format_start_date(date_start)

    if date_end is not None:
        sap_experience["endDate"] = format_end_date(date_end)

    return sap_experience


def format_profile(profile: HrFlowProfile) -> t.Dict:
    """
    Formats an hrflow profile into a sap successfactors candidate
    Args:
        profile (HrflowProfile): profile to format
    Returns:
        SapCandidateModel: a SAP successfactors profile
    """
    sap_profile = dict()

    # Basic information
    info = profile.get("info")
    sap_profile["country"] = DEFAULT_COUNTRY
    sap_profile["address"] = info.get("location").get("text")
    sap_profile["cellPhone"] = info.get("phone")
    fields = info.get("location").get("fields")
    if fields not in [None, []]:
        sap_profile["country"] = fields.get("country")[:-1]
        sap_profile["city"] = fields.get("city")
        sap_profile["zip"] = fields.get("postcode")

    sap_profile["primaryEmail"] = info.get("email")
    sap_profile["firstName"] = info.get("first_name")
    sap_profile["lastName"] = info.get("last_name")
    sap_profile["currentTitle"] = info.get("summary")

    # educations
    if profile.get("educations") is not None:
        sap_profile["education"] = dict(results=[])
        for education in profile.get("educations", []):
            sap_education = format_education(education)
            sap_profile["education"]["results"].append(sap_education)

    # experiences
    if profile.get("experiences") is not None:
        sap_profile["outsideWorkExperience"] = dict(results=[])
        for experience in profile.get("experiences", []):
            sap_experience = format_experience(experience)
            sap_profile["outsideWorkExperience"]["results"].append(sap_experience)
    return sap_profile


DESCRIPTION = (
    "By understanding what employees need, how they work, and what motivates them, you"
    " can put people at the heart of your HR strategy."
)

SAPSuccessFactors = Connector(
    name="SAPSuccessFactors",
    description=DESCRIPTION,
    url="https://www.sap.com/",
    actions=[
        ConnectorAction(
            name="pull_jobs",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs via the ***SAPSuccessFactors*** API and sends them"
                " to a ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadJobsActionParameters", format=format_job
            ),
            origin=SAPJobWarehouse,
            target=HrFlowJobWarehouse,
        ),
        ConnectorAction(
            name="push_profile",
            trigger_type=WorkflowType.catch,
            description=(
                "Writes a profile taken from a Hrflow.ai Source to SAPSuccessFactors"
                " via the SAP API"
            ),
            parameters=BaseActionParameters.with_defaults(
                "WriteProfileActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=SAPProfileWarehouse,
        ),
    ],
)



from typing import Optional

from pydantic import BaseModel, Field


class Location(BaseModel):
    location_str: Optional[str]
    country: Optional[str]
    country_code: Optional[str]
    region: Optional[str]
    region_code: Optional[str]
    city: Optional[str]
    zip_code: Optional[str]
    telecommuting: Optional[bool]


class WorkableJobModel(BaseModel):
    id: Optional[str]
    title: str
    full_title: str
    shortcode: str
    code: Optional[str]
    state: Optional[str]
    department: Optional[str]
    url: Optional[str]
    application_url: Optional[str]
    shortlink: Optional[str]
    location: Location
    created_at: str
    description: Optional[str]
    requirements: Optional[str]
    benefit: Optional[str]
    employment_type: Optional[str]


class Profile(BaseModel):
    name: str
    summary: Optional[str]
    address: Optional[str]
    headline: Optional[str] = Field(
        None, description="One line description as provided by the candidate or you"
    )
    phone: Optional[str]
    email: str
    resume_url: Optional[str]


class WorkableCandidate(BaseModel):
    sourced: Optional[bool] = True
    candidate: Profile



import re

from hrflow_connectors.connectors.breezyhr.utils.datetime_converter import (
    from_str_to_datetime,
)
from hrflow_connectors.connectors.breezyhr.utils.remove_html_tags import (
    remove_html_tags,
)
from hrflow_connectors.connectors.breezyhr.warehouse import (
    BreezyHRJobWarehouse,
    BreezyHRProfileWarehouse,
)
from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.core.connector import (
    BaseActionParameters,
    Connector,
    ConnectorAction,
    WorkflowType,
)

from ..hrflow.schemas import HrFlowJob, HrFlowProfile
from .schemas import BreezyJobModel, BreezyProfileModel


def format_jobs(breezy_job: BreezyJobModel) -> HrFlowJob:
    """
    Format a Breezy Hr job object into a hrflow job object
    Returns:
        HrflowJob: a job object in the hrflow job format
    """
    hrflow_job = dict()
    # Basic information
    hrflow_job["name"] = breezy_job.get("name")
    hrflow_job["reference"] = breezy_job.get("friendly_id")
    hrflow_job["summary"] = None

    # Location
    location = breezy_job.get("location")
    country = location.get("country")
    country_name = country.get("name")
    city = location.get("city")
    address = location.get("name")
    geojson = dict(country=country_name, city=city)

    hrflow_job["location"] = dict(text=address, geojson=geojson, lat=None, lng=None)

    # Sections
    description = remove_html_tags(breezy_job.get("description"))
    cleaned_description = description.replace("&nbsp;", " ")
    hrflow_job["sections"] = [
        dict(
            name="breezy_hr_description",
            title="Breezy_hr_description",
            description=cleaned_description,
        )
    ]
    # tags
    hrflow_job["tags"] = []

    def create_tag(field_name: str):
        tag_name = f"breezy_hr_{field_name}"
        tag_value = breezy_job.get(field_name)

        if isinstance(tag_value, dict):
            tag_name_value = tag_value.get("name")
            tag = dict(name=tag_name, value=tag_name_value)
            hrflow_job["tags"].append(tag)
        if isinstance(tag_value, str):
            tag = dict(name=tag_name, value=tag_value)
            hrflow_job["tags"].append(tag)

    create_tag("type")
    create_tag("experience")
    create_tag("education")
    create_tag("department")
    create_tag("requisition_id")
    create_tag("category")
    create_tag("candidate_type")
    is_remote = dict(name="breezy_hr_remote", value=location.get("is_remote"))
    hrflow_job["tags"].append(is_remote)

    hrflow_job["created_at"] = breezy_job.get("creation_date")
    hrflow_job["updated_at"] = breezy_job.get("updated_date")

    return hrflow_job


def format_profile(hrflow_profile: HrFlowProfile) -> BreezyProfileModel:
    """
    Format a Hrflow profile object into a breezy hr profile object
    Args:
        data (HrflowProfile): Hrflow Profile to format
    Returns:
        BreezyProfileModel: a BreezyHr formatted profile object
    """

    breezy_profile = dict()
    info = hrflow_profile.get("info")
    breezy_profile["name"] = info.get("full_name")
    breezy_profile["address"] = info.get("location").get("text")
    breezy_profile["email_address"] = info.get("email")
    breezy_profile["phone_number"] = info.get("phone")
    breezy_profile["summary"] = info.get("summary")
    breezy_profile["work_history"] = []

    def format_experiences():
        experiences = hrflow_profile.get("experiences")
        for experience in experiences:
            format_experience = dict()
            if experience.get("company") not in ["", None]:
                format_experience["company_name"] = experience.get("company")
            else:
                format_experience["company_name"] = "Undefined"
            format_experience["title"] = experience.get("title")
            format_experience["summary"] = experience.get("description")
            if experience.get("date_start") is not None:
                date_iso = from_str_to_datetime((experience.get("date_start")))
                format_experience["start_year"] = date_iso.year
                format_experience["start_month"] = date_iso.month
            if experience.get("date_end") is not None:
                date_end_iso = from_str_to_datetime((experience.get("date_end")))
                format_experience["end_year"] = date_end_iso.year
                format_experience["end_month"] = date_end_iso.month

            breezy_profile["work_history"].append(format_experience)

    format_experiences()

    breezy_profile["education"] = []

    def format_educations():
        educations = hrflow_profile.get("educations")
        for education in educations:
            format_education = dict()
            if education.get("school") == "":
                education["school"] = "Undefined"
            format_education["school_name"] = education.get("school")
            format_education["field_of_study"] = education.get("title")
            if education.get("date_start") is not None:
                date_iso = from_str_to_datetime((education.get("date_start")))
                format_education["start_year"] = date_iso.year
            if education.get("date_end") is not None:
                date_end_iso = from_str_to_datetime((education.get("date_end")))
                format_education["end_year"] = date_end_iso.year
            breezy_profile["education"].append(format_education)

    format_educations()

    breezy_profile["social_profiles"] = {}

    def format_urls() -> None:
        """
        format_urls, add links and websites to Taleez profile Social links
        """
        urls = info.get("urls")
        if isinstance(urls, list):
            for url in urls:
                try:
                    type = url.get("type")
                    link = url.get("url")
                    if isinstance(link, str):
                        if not re.match(r"http(s)?:\/\/.*", link):
                            # To bypass Breezy invalid_url when is valid_url
                            link = "https://" + link
                        breezy_profile.get("social_profiles").update({type: link})
                except Exception:
                    continue
        attachments = info.get("attachments")
        if isinstance(attachments, list):
            for attachment in attachments:
                file_name = attachment.get("file_name")
                public_url = attachment.get("public_url")
                if isinstance(public_url, str):
                    breezy_profile.get("social_profiles").update(
                        {file_name: public_url}
                    )

    format_urls()

    # add profile skills to tags
    breezy_profile["tags"] = []
    skills = hrflow_profile.get("skills")
    if isinstance(skills, list):
        for skill in skills:
            if isinstance(skill, dict):
                breezy_profile["tags"].append(skill.get("name"))

    return breezy_profile


BreezyHR = Connector(
    name="BreezyHR",
    description=(
        "Breezyhr is an end-to-end recruiting software "
        "to help you attract & hire great employees with less effort"
    ),
    url="https://breezy.hr/",
    actions=[
        ConnectorAction(
            name="pull_jobs",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs via the ***BreezyHR*** API and send them"
                " to a ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "PullJobsActionParameters", format=format_jobs
            ),
            origin=BreezyHRJobWarehouse,
            target=HrFlowJobWarehouse,
        ),
        ConnectorAction(
            name="push_profiles",
            trigger_type=WorkflowType.catch,
            description=(
                "Push all profiles from ***Hrflow.ai Source*** via ***BreezyHR*** API"
                " and send them to a ***BreezyHR***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "PushProfilesActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=BreezyHRProfileWarehouse,
        ),
    ],
)



import json
import typing as t
from logging import LoggerAdapter

import requests
from pydantic import Field

from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
)

from .schemas import BreezyJobModel


class CompanyId(ParametersModel):
    company_id: t.Optional[str] = Field(
        None,
        description=(
            "[⚠️ Requiered if company_name is not specified], ID of company to pull"
            " jobs from in Breezy HR database associated with the authenticated user"
        ),
        field_type=FieldType.Other,
    )


class CompanyName(ParametersModel):
    company_name: t.Optional[str] = Field(
        None,
        description=(
            "[⚠️ Requiered if company_id is not specified], the company associated with"
            " the authenticated user"
        ),
        field_type=FieldType.Other,
    )


class BreezyhrReadParameters(ParametersModel):
    email: str = Field(..., description="email", field_type=FieldType.Other)

    password: str = Field(..., description="password", field_type=FieldType.Auth)

    company_id: t.Optional[str] = Field(
        None,
        description=(
            "ID of company to pull jobs from in Breezy HR database associated with the"
            " authenticated user"
        ),
        field_type=FieldType.Other,
    )
    company_name: t.Optional[str] = Field(
        None,
        description=(
            "[⚠️ Requiered if company_id is not specified], the company associated with"
            " the authenticated user"
        ),
        field_type=FieldType.Other,
    )


class BreezyHRWriteParameters(ParametersModel):
    email: str = Field(..., description="email", field_type=FieldType.Other)

    password: str = Field(..., description="password", field_type=FieldType.Auth)

    company_id: t.Optional[str] = Field(
        None,
        description=(
            "ID of company to pull jobs from in Breezy HR database associated with the"
            " authenticated user \n [⚠️ Requiered if company_name is not specified]"
        ),
        field_type=FieldType.Other,
    )
    company_name: t.Optional[str] = Field(
        None,
        description=(
            "the company associated with the authenticated user \n [⚠️ Requiered if"
            " company_id is not specified]"
        ),
        field_type=FieldType.Other,
    )

    position_id: str = Field(
        ...,
        description="Id of the position to create a new candidate for",
        field_type=FieldType.Other,
    )
    origin: t.Optional[str] = Field(
        "sourced",
        description=(
            "will indicate in Breezy if the candidate should be marked as sourced or"
            " applied"
        ),
        field_type=FieldType.Other,
    )


def get_access_token(
    adapter: LoggerAdapter,
    parameters: BreezyhrReadParameters,
):
    url = "https://api.breezy.hr/v3/signin"

    payload = json.dumps(
        {"email": f"{parameters.email}", "password": f"{parameters.password}"}
    )
    headers = {"Content-Type": "application/json"}

    response = requests.post(url, headers=headers, data=payload)

    if not response.ok:
        adapter.error(f"Fail to get token, reason: {response.text}")
        raise f"Fail to get token, reason: {response.text}"

    return response.json().get("access_token")


def revoke_access_token(access_token):
    url = "https://api.breezy.hr/v3/signout"

    payload = {}
    headers = {"Content-Type": "application/json", "Authorization": f"{access_token}"}

    requests.get(url, headers=headers, data=payload)


def get_compagnie_id(
    adapter: LoggerAdapter, parameters: BreezyhrReadParameters, access_token: str
):

    if parameters.company_id is not None:
        return parameters.company_id

    url = "https://api.breezy.hr/v3/companies"

    payload = {}
    headers = {"Content-Type": "application/json", "Authorization": f"{access_token}"}

    response = requests.get(url, headers=headers, data=payload)

    if not response.ok:
        adapter.error(f"Fail to get compagnie id, reason: {response.text}")
        raise f"Fail to get compagnie id, reason: {response.text}"

    company_list = response.json()
    for company in company_list:
        if company["name"] == parameters.company_name:
            return company["_id"]

    adapter.error(
        "Fail to get compagnie id, reason: compagny_name does not match with an id"
    )
    raise "Fail to get compagnie id, reason: compagny_name does not match with an id"


def read(
    adapter: LoggerAdapter,
    parameters: BreezyhrReadParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterator[BreezyJobModel]:
    access_token = get_access_token(adapter, parameters)
    compagnie_id = get_compagnie_id(adapter, parameters, access_token)

    url = f"https://api.breezy.hr/v3/company/{compagnie_id}/positions?state=published"

    headers = {"Content-Type": "application/json", "Authorization": f"{access_token}"}

    response = requests.get(url, headers=headers)
    if not response.ok:
        adapter.error(f"Fail to read, reason: {response.text}")
        raise adapter.error(f"Fail to read, reason: {response.text}")

    for job in response.json():
        yield job

    revoke_access_token(access_token)


def send_profile(
    adapter: LoggerAdapter,
    parameters: BreezyHRWriteParameters,
    profiles: t.Iterable[t.Dict],
    access_token: str,
    company_id: str,
    candidate_id: str = "",
):

    base_url = (
        f"https://api.breezy.hr/v3/company/{company_id}/"
        f"position/{parameters.position_id}/candidate"
    )

    payload = json.dumps(profiles)

    headers = {"Content-Type": "application/json", "Authorization": f"{access_token}"}

    if candidate_id == "":

        url = f"{base_url}s/"

        # If the candidate doesn't already exist we "POST" his profile
        response = requests.post(url, headers=headers, data=payload)

    else:
        # In case the candidate exists,
        #  we retrieve his id to update his profile with a "PUT" request

        url = f"{base_url}/{candidate_id}"

        url = (
            f"https://api.breezy.hr/v3/company/{company_id}/"
            f"position/{parameters.position_id}/candidate/{candidate_id}"
        )

        adapter.info(f"Updating id = {candidate_id} profile")
        response = requests.put(url, headers=headers, data=payload)
    return response


def write(
    adapter: LoggerAdapter,
    parameters: BreezyHRWriteParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:

    failed_profiles = []
    access_token = get_access_token(adapter, parameters)
    compagnie_id = get_compagnie_id(adapter, parameters, access_token)

    for profile in profiles:

        profile.update({"origin": parameters.origin})

        mail = profile.get("email_address")

        url = (
            f"https://api.breezy.hr/v3/company/{compagnie_id}/"
            f"candidates/search?email_address={mail}"
        )

        headers = {
            "Content-Type": "application/json",
            "Authorization": f"{access_token}",
        }

        response = requests.get(url, headers=headers)
        if not response.ok:
            adapter.error(f"Couldn't get candidate {profile}")
            failed_profiles.append(profile)
            continue

        candidate_list = response.json()
        candidate_id = ""

        # In case the candidate exists
        # we retrieve his id to update his profile with a "PUT" request
        if candidate_list != []:
            candidate_id = candidate_list[0].get("_id")
            adapter.info(f"Candidate Already exists with the id {candidate_id}")

        response = send_profile(
            adapter, parameters, profile, access_token, compagnie_id, candidate_id
        )
        if not response.ok:
            adapter.error(f"Fail to post user : {profile}")
            failed_profiles.append(profile)

    revoke_access_token(access_token)
    return failed_profiles


BreezyHRJobWarehouse = Warehouse(
    name="BreezyHRJobWarehouse",
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=BreezyhrReadParameters,
        function=read,
    ),
)

BreezyHRProfileWarehouse = Warehouse(
    name="BreezyHRWarehouse",
    data_type=DataType.profile,
    write=WarehouseReadAction(
        parameters=BreezyHRWriteParameters,
        function=write,
    ),
)



import typing as t

from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.recruitee.warehouse import (
    RecruiteeJobWarehouse,
    RecruiteeProfileWarehouse,
)
from hrflow_connectors.core import (
    BaseActionParameters,
    Connector,
    ConnectorAction,
    WorkflowType,
)


def get_profile_cv_url(attachments: t.List[t.Dict]):
    cv_url = next(
        (attachment for attachment in attachments if attachment.get("type") == "resume")
    )["public_url"]
    return cv_url


def get_profile_links(urls: t.List[t.Dict]) -> t.List:
    profile_links = []
    social_links = []

    profile_links = [e["url"] for e in urls if e.get("type") == "from_resume"]
    social_links = [e["url"] for e in urls if e.get("type") != "from_resume"]

    return profile_links, social_links


def format_profile(hrflow_profile: t.Dict) -> t.Dict:
    hrflow_profile_info = hrflow_profile["info"]
    profile_links, social_links = get_profile_links(hrflow_profile_info["urls"])
    profile = dict(
        name=hrflow_profile_info["full_name"],
        remote_cv_url=get_profile_cv_url(hrflow_profile["attachments"]),
        emails=[hrflow_profile_info["email"]],
        phones=[hrflow_profile_info["phone"]],
        social_links=social_links,
        links=profile_links,
        cover_letter="",
        sources=[hrflow_profile["source"]["name"]],
    )
    return profile


def get_tags(recruitee_job: t.Dict) -> t.List[t.Dict]:
    job = recruitee_job
    t = lambda name, value: dict(name=name, value=value)

    tags = [
        t("recruitee_category", job.get("category")),
        t("recruitee_department", job.get("department")),
        t("recruitee_options_cv", job.get("options_cv")),
        t("recruitee_options_cover_letter", job.get("options_cover_letter")),
        t("recruitee_experience", job.get("experience")),
        t("recruitee_education", job.get("education")),
        t("recruitee_employment_type", job.get("employment_type")),
        t("recruitee_remote_option", job.get("remote")),
        t("recruitee_candidates_count", job.get("candidates_count")),
        t(
            "recruitee_disqualified_candidates_count",
            job.get("disqualified_candidates_count"),
        ),
        t(
            "recruitee_qualified_candidates_count",
            job.get("qualified_candidates_count"),
        ),
        t("recruitee_hired_candidates_count", job.get("hired_candidates_count")),
    ]
    return tags


def get_ranges_float(recruitee_job: t.Dict) -> t.List[t.Dict]:
    t = lambda name, value_min, value_max, unit: dict(
        name=name, value_min=value_min, value_max=value_max, unit=unit
    )
    salary = recruitee_job.get("salary", {})
    ranges_float = [
        t(
            "working hours",
            recruitee_job.get("min_hours"),
            recruitee_job.get("max_hours"),
            "Hours per week",
        ),
        t(
            "salary per {}".format(salary.get("period")),
            salary.get("min"),
            salary.get("max"),
            salary.get("currency"),
        ),
    ]
    return ranges_float


def format_job(recruitee_job: t.Dict) -> t.Dict:
    sections = [
        dict(
            name="recruitee_job_requirements",
            title="Job Requirements",
            description=recruitee_job["requirements"],
        )
    ]
    job = dict(
        name=recruitee_job.get("title"),
        reference=str(recruitee_job.get("id")),
        created_at=recruitee_job.get("created_at"),
        updated_at=recruitee_job.get("updated_at"),
        location=dict(lat=None, lng=None, text=recruitee_job.get("location")),
        url=recruitee_job.get("url"),
        summary=recruitee_job.get("description"),
        sections=sections,
        tags=get_tags(recruitee_job),
        ranges_float=get_ranges_float(recruitee_job),
    )
    return job


Recruitee = Connector(
    name="Recruitee",
    description="",
    url="https://recruitee.com/",
    actions=[
        ConnectorAction(
            name="push_profile",
            trigger_type=WorkflowType.catch,
            description=(
                "Writes a profile from Hrflow.ai Source as a candidate on Recruitee via"
                " the API"
            ),
            parameters=BaseActionParameters.with_defaults(
                "WriteProfileActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=RecruiteeProfileWarehouse,
        ),
        ConnectorAction(
            name="pull_jobs",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs via the ***Recruitee*** API and send them"
                " to a ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadJobsActionParameters", format=format_job
            ),
            origin=RecruiteeJobWarehouse,
            target=HrFlowJobWarehouse,
        ),
    ],
)



from jinja2 import Template

CONNECTOR_README_TEMPLATE = Template(
    """
# {{ connector_name }} Connector
{% if description %}
> {{ description }}
{% endif %}

🔗 {{ url }}

| Actions |
| ------- |
{% for action in actions %}| [**{{ action.name | title | replace("_", " ")}}**](docs/{{ action.name }}.md) |
{% endfor %}
"""
)
ACTION_DOCUMENTATION_TEMAPLTE = Template(
    """
# {{ action_name | title | replace("_", " ") }}
`{{ origin_name }}` :arrow_right: `{{ target_name }}`

{{ description }}

{% if origin_endpoints %}
**{{ origin_name }} endpoints used :**
| Endpoints | Description |
| --------- | ----------- |
{%- for endpoint in origin_endpoints %}
| [**{{ endpoint.name}}**]({{ endpoint.url }}) | {{ endpoint.description }} |
{%- endfor %}

{% endif %}
{% if target_endpoints %}
**{{ target_name }} endpoints used :**
| Endpoints | Description |
| --------- | ----------- |
{%- for endpoint in target_endpoints %}
| [**{{ endpoint.name}}**]({{ endpoint.url }}) | {{ endpoint.description }} |
{%- endfor %}

{% endif %}
## Action Parameters

| Field | Type | Default | Description |
| ----- | ---- | ------- | ----------- |
{%- for field in action_fields %}
| `{{ field.name }}` {% if field.required %}:red_circle:{% endif %} | `{{ field.type }}` | {{ field.default }} | {{ field.description }} |
{%- endfor %}

## Source Parameters

| Field | Type | Default | Description |
| ----- | ---- | ------- | ----------- |
{%- for field in origin_fields %}
| `{{ field.name }}` {% if field.required %}:red_circle:{% endif %} | `{{ field.type }}` | {{ field.default }} | {{ field.description }} |
{%- endfor %}

## Destination Parameters

| Field | Type | Default | Description |
| ----- | ---- | ------- | ----------- |
{%- for field in target_fields %}
| `{{ field.name }}` {% if field.required %}:red_circle:{% endif %} | `{{ field.type }}` | {{ field.default }} | {{ field.description }} |
{%- endfor %}

:red_circle: : *required*

## Example

```python
import logging
from hrflow_connectors import {{ connector_name }}
from hrflow_connectors.core import ReadMode


logging.basicConfig(level=logging.INFO)


{{ connector_name }}.{{ action_name }}(
    workflow_id="some_string_identifier",
    action_parameters=dict(
        {%- for field in action_fields %}
        {{ field.name }}={{ field.example }},
        {%- endfor %}
    ),
    origin_parameters=dict(
        {%- for field in origin_fields %}
        {{ field.name }}={{ field.example }},
        {%- endfor %}
    ),
    target_parameters=dict(
        {%- for field in target_fields %}
        {{ field.name }}={{ field.example }},
        {%- endfor %}
    )
)
```
"""
)

WORKFLOW_TEMPLATE = Template(
    """
import typing as t

from hrflow_connectors import {{ connector_name }}
from hrflow_connectors.core.connector import ActionInitError, Reason

ORIGIN_SETTINGS_PREFIX = "{{ origin_settings_prefix }}"
TARGET_SETTINGS_PREFIX = "{{ target_settings_prefix }}"

{{ format_placeholder }}

{{ logics_placeholder }}
{% if type == "catch" %}
{{ event_parser_placeholder }}
{% endif %}

def workflow(
        {% if type == "catch" %}
        _request: t.Dict,
        {% endif %}
        settings: t.Dict
    ) -> None:
    actions_parameters = dict()
    try:
        format
    except NameError:
        pass
    else:
        actions_parameters["format"] = format

    try:
        logics
    except NameError:
        pass
    else:
        actions_parameters["logics"] = logics

    if "{{ workflow_id_settings_key }}" not in settings:
        return {{ connector_name }}.{{ action_name }}(
            workflow_id="",
            action_parameters=dict(),
            origin_parameters=dict(),
            target_parameters=dict(),
            init_error=ActionInitError(
                reason=Reason.workflow_id_not_found,
                data=dict(error="{{ workflow_id_settings_key }} not found in settings", settings_keys=list(settings.keys())),
            )
        )
    workflow_id = settings["{{ workflow_id_settings_key }}"]

    {% if type == "catch" %}
    try:
        event_parser
        _event_parser = event_parser
    except NameError as e:
        action = {{ connector_name }}.model.action_by_name("{{ action_name }}")
        # Without this trick event_parser is always only fetched from the local scope
        # meaning that try block always raises NameError even if the function is
        # defined in the placeholder
        _event_parser = action.parameters.__fields__["event_parser"].default

    if _event_parser is not None:
        try:
            _request = _event_parser(_request)
        except Exception as e:
            return {{ connector_name }}.{{ action_name }}(
                workflow_id=workflow_id,
                action_parameters=dict(),
                origin_parameters=dict(),
                target_parameters=dict(),
                init_error=ActionInitError(
                    reason=Reason.event_parsing_failure,
                    data=dict(error=e, event=_request),
                )
            )
    {% endif %}

    origin_parameters = dict()
    for parameter in {{ origin_parameters }}:
        if "{}{}".format(ORIGIN_SETTINGS_PREFIX, parameter) in settings:
            origin_parameters[parameter] = settings["{}{}".format(ORIGIN_SETTINGS_PREFIX, parameter)]
        {% if type == "catch" %}
        if parameter in _request:
            origin_parameters[parameter] = _request[parameter]
        {% endif %}

    target_parameters = dict()
    for parameter in {{ target_parameters }}:
        if "{}{}".format(TARGET_SETTINGS_PREFIX, parameter) in settings:
            target_parameters[parameter] = settings["{}{}".format(TARGET_SETTINGS_PREFIX, parameter)]
        {% if type == "catch" %}
        if parameter in _request:
            target_parameters[parameter] = _request[parameter]
        {% endif %}

    return {{ connector_name }}.{{ action_name }}(
        workflow_id=workflow_id,
        action_parameters=actions_parameters,
        origin_parameters=origin_parameters,
        target_parameters=target_parameters,
    )
"""
)



from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field


class Type(BaseModel):
    id: str
    name: str


class Experience(BaseModel):
    id: str
    name: str


class Country(BaseModel):
    name: str
    id: str


class Location(BaseModel):
    country: Country
    city: str
    is_remote: Optional[bool]
    name: str


class Category(BaseModel):
    id: str
    name: str


class ApplicationForm(BaseModel):
    name: str
    headline: str
    summary: str
    profile_photo: str
    address: str
    email_address: str
    phone_number: str
    resume: str
    work_history: str
    education: str
    cover_letter: str
    questionnaire_in_experience: bool


class BreezyJobModel(BaseModel):
    _id: str = Field(..., description="position id")
    type: Type = Field(..., description="job type")
    state: str = Field(
        ...,
        description="state of the position posting, published or internal and so on",
    )
    name: str = Field(..., description="job title")
    friendly_id: str = Field(
        ..., description="another id of the job which combines its title and its id"
    )
    experience: Optional[Experience]
    location: Location
    education: str
    department: str
    description: str = Field(..., description="Job category")
    category: Category
    application_form: Optional[ApplicationForm] = Field(
        ..., description="job Application for"
    )
    creator_id: Optional[str]
    creation_date: str
    updated_date: str
    all_users: List[str]
    all_admins: List[str]
    candidate_type: str
    tags: List
    org_type: str


class WorkHistoryItem(BaseModel):
    company_name: str
    title: str
    summary: str
    start_month: Optional[int]
    start_year: Optional[int]
    end_month: Optional[int]
    end_year: Optional[int]


class EducationItem(BaseModel):
    school_name: str
    field_of_study: str
    start_year: Optional[int]
    end_year: Optional[int]


class BreezyProfileModel(BaseModel):
    name: str
    email_address: str
    phone_number: str
    summary: str
    tags: Optional[List[str]]
    source: Optional[str]
    origin: Optional[str]
    address: str
    work_history: List[WorkHistoryItem]
    education: List[EducationItem]
    social_profiles: Optional[List[str]]
    custom_attributes: Optional[List[Dict[str, Any]]]
    cover_letter: Optional[str]



import enum
import logging
import os
import typing as t
from pathlib import Path

from pydantic import BaseModel
from pydantic.fields import ModelField

from hrflow_connectors.core.connector import Connector
from hrflow_connectors.core.templates import (
    ACTION_DOCUMENTATION_TEMAPLTE,
    CONNECTOR_README_TEMPLATE,
)

logger = logging.getLogger(__name__)
CONNECTORS_DIRECTORY = Path(__file__).parent.parent / "connectors"


class TemplateField(BaseModel):
    name: str
    type: str
    required: bool
    description: str
    example: str
    default: str


def field_example(field: ModelField) -> str:
    if callable(field.default):
        return "lambda *args, **kwargs: None # Put your code logic here"

    if field.default is not None:
        if isinstance(field.default, str):
            return '"{}"'.format(field.default)
        return str(field.default)

    if field.default_factory is not None:
        return str(field.default_factory())

    field_type = field.outer_type_
    if isinstance(field_type, enum.EnumMeta):
        return '"{}"'.format(list(field_type)[0].value)

    if field_type is str:
        return '"your_{}"'.format(field.name)

    if field_type in [int, float, bool]:
        return str(field_type())

    return "***"


def field_default(field: ModelField, documentation_path: Path) -> str:
    if callable(field.default):
        relative_filepath = os.path.relpath(
            field.default.__code__.co_filename, documentation_path
        )
        return "[`{}`]({}#L{})".format(
            field.default.__code__.co_name,
            relative_filepath,
            field.default.__code__.co_firstlineno,
        )

    if field.default_factory is not None:
        return str(field.default_factory())

    return str(field.default)


def field_type(field: ModelField) -> str:
    if isinstance(field.outer_type_, enum.EnumMeta):
        return "str"
    if field.type_.__name__ == "Callable":
        return str(field.outer_type_)
    return field.type_.__name__


def get_template_fields(
    fields: t.List[ModelField], documentation_path: Path
) -> t.List[TemplateField]:
    return [
        TemplateField(
            name=field.name,
            type=field_type(field),
            required=field.required,
            description=field.field_info.description or "",
            example=field_example(field),
            default=field_default(field, documentation_path),
        )
        for field in fields
        if not field.field_info.const
        and field.field_info.extra.get("skip_from_docs", False) is False
    ]


def generate_docs(
    connectors: t.List[Connector], connectors_directory: Path = CONNECTORS_DIRECTORY
) -> None:
    for connector in connectors:
        model = connector.model
        connector_directory = connectors_directory / model.name.lower()
        if not connector_directory.is_dir():
            logging.error(
                "Skipping documentation for {}: no directory found at {}".format(
                    model.name, connector_directory
                )
            )
            continue
        readme_content = CONNECTOR_README_TEMPLATE.render(
            connector_name=model.name.capitalize(),
            description=model.description,
            url=model.url,
            actions=model.actions,
        )
        readme = connector_directory / "README.md"
        readme.write_bytes(readme_content.encode())
        if len(model.actions) > 0:
            action_docs_directory = connector_directory / "docs"
            if not action_docs_directory.is_dir():
                action_docs_directory.mkdir()
            for action in model.actions:
                action_fields = get_template_fields(
                    fields=action.parameters.__fields__.values(),
                    documentation_path=action_docs_directory,
                )
                origin_fields = get_template_fields(
                    fields=action.origin.read.parameters.__fields__.values(),
                    documentation_path=action_docs_directory,
                )
                target_fields = get_template_fields(
                    fields=action.target.write.parameters.__fields__.values(),
                    documentation_path=action_docs_directory,
                )
                action_documentation_content = ACTION_DOCUMENTATION_TEMAPLTE.render(
                    connector_name=model.name,
                    action_name=action.name,
                    description=action.description,
                    action_fields=action_fields,
                    origin_name=action.origin.name,
                    origin_fields=origin_fields,
                    origin_endpoints=action.origin.read.endpoints,
                    target_name=action.target.name,
                    target_fields=target_fields,
                    target_endpoints=action.target.write.endpoints,
                )
                action_documentation = action_docs_directory / "{}.md".format(
                    action.name
                )
                action_documentation.write_bytes(action_documentation_content.encode())



import typing as t

from pydantic import BaseModel, Field


# Common
class Location(BaseModel):
    text: t.Optional[str] = Field(None, description="Location text address.")
    lat: t.Optional[float] = Field(
        None, description="Geocentric latitude of the Location."
    )
    lng: t.Optional[float] = Field(
        None, description="Geocentric longitude of the Location."
    )
    _fields: t.Optional[t.Dict[str, t.Any]] = Field(
        None,
        alias="fields",
        description="other location attributes like country, country_codeNoneetc",
    )


class GeneralEntitySchema(BaseModel):
    name: str = Field(..., description="Identification name of the Object")
    value: t.Optional[str] = Field(
        None, description="Value associated to the Object's name"
    )


class Skill(BaseModel):
    name: str = Field(..., description="Identification name of the skill")
    type: t.Optional[str] = Field(None, description="Type of the skill. hard or soft")
    value: t.Optional[str] = Field(None, description="Value associated to the skill")


# Job
class Section(BaseModel):
    name: t.Optional[str] = Field(
        None,
        description="Identification name of a Section of the Job. Example: culture",
    )
    title: t.Optional[str] = Field(
        None, description="Display Title of a Section. Example: Corporate Culture"
    )
    description: t.Optional[str] = Field(
        None, description="Text description of a Section: Example: Our values areNone"
    )


class RangesFloat(BaseModel):
    name: t.Optional[str] = Field(
        None,
        description=(
            "Identification name of a Range of floats attached "
            "to the Job. Example: salary"
        ),
    )
    value_min: t.Optional[float] = Field(None, description="Min value. Example: 500.")
    value_max: t.Optional[float] = Field(None, description="Max value. Example: 100.")
    unit: t.Optional[str] = Field(
        None, description="Unit of the value. Example: euros."
    )


class RangesDate(BaseModel):
    name: t.Optional[str] = Field(
        None,
        description=(
            "Identification name of a Range of dates attached"
            " to the Job. Example: availability."
        ),
    )
    value_min: t.Optional[str] = Field(
        None, description="Min value in datetime ISO 8601, Example: 500."
    )
    value_max: t.Optional[str] = Field(
        None, description="Max value in datetime ISO 8601, Example: 1000"
    )


class HrFlowJob(BaseModel):
    key: t.Optional[str] = Field(None, description="Identification key of the Job.")
    reference: t.Optional[str] = Field(
        None, description="Custom identifier of the Job."
    )
    name: str = Field(..., description="Job title.")
    location: Location = Field(None, description="Job location object.")
    sections: t.List[Section] = Field(None, description="Job custom sections.")
    url: t.Optional[str] = Field(None, description="Job post original URL.")
    summary: t.Optional[str] = Field(None, description="Brief summary of the Job.")
    archieved_at: t.Optional[str] = Field(
        None,
        description=(
            "type: datetime ISO8601, Archive date of the Job. "
            "The value is null for unarchived Jobs."
        ),
    )
    updated_at: t.Optional[str] = Field(
        None, description="type: datetime ISO8601, Last update date of the Job."
    )
    created_at: t.Optional[str] = Field(
        None, description="type: datetime ISO8601, Creation date of the Job."
    )
    skills: t.Optional[t.List[Skill]] = Field(
        None, description="t.List of skills of the Job."
    )
    languages: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="t.List of spoken languages of the Job"
    )
    certifications: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="t.List of certifications of the Job."
    )
    courses: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="t.List of courses of the Job"
    )
    tasks: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="t.List of tasks of the Job"
    )
    tags: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="t.List of tags of the Job"
    )
    metadatas: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="t.List of metadatas of the Job"
    )
    ranges_float: t.Optional[t.List[RangesFloat]] = Field(
        None, description="t.List of ranges of floats"
    )
    ranges_date: t.Optional[t.List[RangesDate]] = Field(
        None, description="t.List of ranges of dates"
    )


# Profile
class InfoUrls(BaseModel):
    from_resume: t.Optional[t.List[str]]
    linkedin: t.Optional[str]
    twitter: t.Optional[str]
    facebook: t.Optional[str]
    github: t.Optional[str]


class ProfileInfo(BaseModel):
    full_name: t.Optional[str]
    first_name: t.Optional[str]
    last_name: t.Optional[str]
    email: t.Optional[str]
    phone: t.Optional[str]
    date_birth: t.Optional[str] = Field(None, description="Profile date of birth")
    location: t.Optional[Location] = Field(None, description="Profile location object")
    urls: t.Optional[InfoUrls] = Field(
        None, description="Profile social networks and URLs"
    )
    picture: t.Optional[str] = Field(None, description="Profile picture url")
    gender: t.Optional[str] = Field(None, description="Profile gender")
    summary: t.Optional[str] = Field(None, description="Profile summary text")


class Experience(BaseModel):
    key: t.Optional[str] = Field(
        None, description="Identification key of the Experience."
    )
    company: t.Optional[str] = Field(
        None, description="Company name of the Experience."
    )
    title: t.Optional[str] = Field(None, description="Title of the Experience.")
    description: t.Optional[str] = Field(
        None, description="Description of the Experience."
    )
    location: t.Optional[Location] = Field(
        None, description="Location object of the Experience."
    )
    date_start: t.Optional[str] = Field(
        None, description="Start date of the experience. type: ('datetime ISO 8601')"
    )
    date_end: t.Optional[str] = Field(
        None, description="End date of the experience. type: ('datetime ISO 8601')"
    )
    skills: t.Optional[t.List[Skill]] = Field(
        None, description="List of skills of the Experience."
    )
    certifications: t.Optional[t.List[GeneralEntitySchema]]
    courses: t.Optional[t.List[GeneralEntitySchema]]
    tasks: t.Optional[t.List[GeneralEntitySchema]]


class Education(BaseModel):
    key: t.Optional[str] = Field(
        None, description="Identification key of the Education."
    )
    school: t.Optional[str] = Field(None, description="School name of the Education.")
    title: t.Optional[str] = Field(None, description="Title of the Education.")
    description: t.Optional[str] = Field(
        None, description="Description of the Education."
    )
    location: t.Optional[Location] = Field(
        None, description="Location object of the Education."
    )
    date_start: t.Optional[str] = Field(
        None, description="Start date of the Education. type: ('datetime ISO 8601')"
    )
    date_end: t.Optional[str] = Field(
        None, description="End date of the Education. type: ('datetime ISO 8601')"
    )
    skills: t.Optional[t.List[Skill]] = Field(
        None, description="List of skills of the Education."
    )
    certifications: t.Optional[t.List[GeneralEntitySchema]]
    courses: t.Optional[t.List[GeneralEntitySchema]]
    tasks: t.Optional[t.List[GeneralEntitySchema]]


class HrFlowProfile(BaseModel):
    key: t.Optional[str] = Field(None, description="Identification key of the Profile.")
    reference: t.Optional[str] = Field(
        None, description="Custom identifier of the Profile."
    )
    archieved_at: t.Optional[str] = Field(
        None,
        description=(
            "type: datetime ISO8601, Archive date of the Profile."
            " The value is null for unarchived Profiles."
        ),
    )
    updated_at: t.Optional[str] = Field(
        None, description="type: datetime ISO8601, Last update date of the Profile."
    )
    created_at: t.Optional[str] = Field(
        None, description="type: datetime ISO8601, Creation date of the Profile."
    )
    info: ProfileInfo = Field(None, description="Object containing the Profile's info.")
    text_language: str = Field(
        None, description="Code language of the Profile. type: string code ISO 639-1"
    )
    text: str = Field(None, description="Full text of the Profile..")
    experiences_duration: float = Field(
        None, description="Total number of years of experience."
    )
    educations_duration: float = Field(
        None, description="Total number of years of education."
    )
    experiences: t.Optional[t.List[Experience]] = Field(
        None, description="List of experiences of the Profile."
    )
    educations: t.Optional[t.List[Education]] = Field(
        None, description="List of educations of the Profile."
    )
    attachments: t.List = Field(
        None, description="List of documents attached to the Profile."
    )
    skills: t.Optional[t.List[Skill]] = Field(
        None, description="List of skills of the Profile."
    )
    languages: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="List of spoken languages of the profile"
    )
    certifications: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="List of certifications of the Profile."
    )
    courses: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="List of courses of the Profile."
    )
    tasks: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="List of tasks of the Profile."
    )
    interests: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="List of interests of the Profile."
    )
    labels: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="List of labels of the Profile."
    )
    tags: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="List of tags of the Profile."
    )
    metadatas: t.Optional[t.List[GeneralEntitySchema]] = Field(
        None, description="List of metadatas of the Profile."
    )


class ResumeToParse(BaseModel):
    raw: bytes
    content_type: str


class HrFlowProfileParsing(BaseModel):
    reference: t.Optional[str] = Field(
        ..., description="Custom identifier of the Profile."
    )
    created_at: str = Field(
        ..., description="type: datetime ISO8601, Creation date of the Profile."
    )
    resume: ResumeToParse
    tags: t.List[GeneralEntitySchema] = Field(
        ..., description="List of tags of the Profile."
    )
    metadatas: t.List[GeneralEntitySchema] = Field(
        ..., description="List of metadatas of the Profile."
    )



import re
import typing as t

from hrflow_connectors.connectors.hrflow.schemas import HrFlowProfile
from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.teamtailor.schema import TeamtailorJob
from hrflow_connectors.connectors.teamtailor.warehouse import (
    TeamtailorJobWarehouse,
    TeamtailorProfileWarehouse,
)
from hrflow_connectors.core import (
    BaseActionParameters,
    Connector,
    ConnectorAction,
    WorkflowType,
)


def remove_html_tags(text: str) -> str:
    """
    Remove all HTML tags in a string
    Args:
        text (str): text to clean
    Returns:
        str: cleaned text (without HTML tags)
    """
    return re.sub("<[^<]+?>", "", text)


# Returns 0 if not a valid float
def str_to_float(value: str) -> float:
    if value.replace(".", "", 1).isdigit():
        return float(value)
    return 0


def format_job(data: TeamtailorJob) -> t.Dict:
    """
    Format a Teamtailor job object into a Hrflow job object
    Args:
        data (TeamtailorJob): Teamtailor job object from the list of jobs pulled
    Returns:
        HrflowJob: a job in the HrFlow job object form
    """
    job = dict()
    job_data = data.get("job").get("data")
    attribute = job_data.get("attributes")
    job["name"] = attribute.get("title")
    job["reference"] = job_data.get("id")
    job["summary"] = attribute.get("pitch")
    job["created_at"] = attribute.get("created-at")
    job["updated_at"] = attribute.get("updated-at")
    job["url"] = job_data.get("links").get("careersite-job-url")

    job_location = data.get("job_location")
    text = job_location.get("text")
    lat = str_to_float(job_location.get("lat"))
    lng = str_to_float(job_location.get("lng"))
    job["location"] = dict(text=text, lat=lat, lng=lng)

    # sections
    description = remove_html_tags(attribute.get("body"))
    job["sections"] = [
        dict(
            name="teamtailor_description",
            title="teamtailor_description",
            description=description,
        )
    ]

    # tags
    t = lambda name, value: dict(name=name, value=value)
    job["tags"] = [
        t("start-date", attribute.get("start-date")),
        t("end-date", attribute.get("end-date")),
        t("status", attribute.get("status")),
        t("employment-type", attribute.get("employment-type")),
        t("employment-level", attribute.get("employment-level")),
        t("remote-status", attribute.get("remote-status")),
        t("salary-time-unit", attribute.get("salary-time-unit")),
        t("min-salary", attribute.get("min-salary")),
        t("max-salary", attribute.get("max-salary")),
        t("currency", attribute.get("currency")),
        t("internal", attribute.get("internal")),
    ]

    return job


def format_profile(data: HrFlowProfile) -> t.Dict:
    """
    Format a Hrflow profile object into a Teamtailor profile object
    Args:
        data (HrflowProfile): Hrflow Profile to format
    Returns:
        Dict[str, Any]: a Teamtailor formatted profile object
    """
    profile = dict()
    info = data.get("info")
    profile["first-name"] = info.get("first_name")
    profile["last-name"] = info.get("last_name")
    profile["email"] = info.get("email")
    profile["phone"] = info.get("phone")
    pitch = info.get("summary")
    profile["pitch"] = pitch[0:139]
    profile["resume"] = ""
    if len(data.get("attachments")) >= 2:
        resume = data.get("attachments")[1]
        profile["resume"] = resume.get("public_url")
    profile["sourced"] = "sourced"
    profile["tags"] = data.get("tags")
    profile_obj = dict()
    profile_obj["data"] = {"type": "candidates", "attributes": profile}
    return profile_obj


DESCRIPTION = (
    "The new way to attract, nurture and hire top talent. Grow faster by focusing on"
    " what matters the most — your candidates"
)

Teamtailor = Connector(
    name="Teamtailor",
    description=DESCRIPTION,
    url="https://www.teamtailor.com/",
    actions=[
        ConnectorAction(
            name="pull_jobs",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieve all jobs via the ***Teamtailor*** API and send them"
                " to an ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadJobsActionParameters", format=format_job
            ),
            origin=TeamtailorJobWarehouse,
            target=HrFlowJobWarehouse,
        ),
        ConnectorAction(
            name="push_profile",
            trigger_type=WorkflowType.catch,
            description=(
                "Writes a profile from an Hrflow.ai Source to Teamtailor via the API"
            ),
            parameters=BaseActionParameters.with_defaults(
                "WriteProfileActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=TeamtailorProfileWarehouse,
        ),
    ],
)



import json
import urllib.parse as urlparse
import urllib.request as urllib2

import requests

base_url = "https://auth.bullhornstaffing.com/oauth"


class AuthCodeRedirectHandler(urllib2.HTTPRedirectHandler):
    """
    A bare bones redirect handler that pulls the auth code sent back
    by OAuth off the query string of the redirect URI given in the
    Location header.  Does no checking for other errors or bad/missing
    information.
    """

    def http_error_302(self, req, fp, code, msg, headers):
        """handler for 302 responses that assumes a properly constructed
        OAuth 302 response and pulls the auth code out of the header"""
        qs = urlparse.urlparse(headers["Location"]).query
        auth_code = urlparse.parse_qs(qs)["code"][0]
        return auth_code


def build_auth_code_request(username, password, client_id):
    data = {
        "client_id": client_id,
        "response_type": "code",
        "username": username,
        "password": password,
        "action": "Login",
    }

    encoded = urlparse.urlencode(data)
    encoded = encoded.encode("utf-8")

    req = urllib2.Request(url=base_url + "/authorize", data=encoded)
    return req


def get_access_token(code, client_id, client_secret):
    """
    Gets an OAuth access token given an OAuth authorization code
    """
    data = {
        "grant_type": "authorization_code",
        "client_id": client_id,
        "client_secret": client_secret,
        "code": code,
    }
    encoded = urlparse.urlencode(data)
    encoded = encoded.encode("utf-8")

    req = urllib2.Request(base_url + "/token", encoded)
    return urllib2.urlopen(req).read()


def auth(username, password, client_id, client_secret):
    req = build_auth_code_request(username, password, client_id)
    opener = urllib2.build_opener(AuthCodeRedirectHandler)

    auth_code = opener.open(req)
    access_token = get_access_token(auth_code, client_id, client_secret)
    access_token = access_token.decode("utf-8")
    access_token = json.loads(access_token)

    login_url = "https://rest.bullhornstaffing.com/rest-services/login"
    params = {"version": "2.0", "access_token": access_token["access_token"]}

    response = requests.post(url=login_url, params=params)
    response = response.json()
    response["refresh_token"] = access_token["refresh_token"]

    return response



import copy
import enum
import json
import logging
import typing as t
import uuid
from collections import Counter
from functools import partial

from pydantic import BaseModel, Field, ValidationError, create_model, validator

from hrflow_connectors.core import backend
from hrflow_connectors.core.templates import WORKFLOW_TEMPLATE
from hrflow_connectors.core.warehouse import ReadMode, Warehouse

logger = logging.getLogger(__name__)


class ConnectorActionAdapter(logging.LoggerAdapter):
    def process(self, msg: str, kwargs: t.Dict) -> t.Tuple[str, t.Dict]:
        tags = [
            "[{}={}]".format(tag["name"], tag["value"])
            for tag in self.extra["log_tags"]
        ]
        return (
            "{}: {}".format(
                "".join(tags),
                msg,
            ),
            kwargs,
        )


class Event(str, enum.Enum):
    read_success = "read_success"
    read_failure = "read_failure"
    format_failure = "format_failure"
    logics_discard = "logics_discard"
    logics_failure = "logics_failure"
    write_failure = "write_failure"
    callback_failure = "callback_failure"
    callback_executed = "callback_executed"
    item_to_read_from_failure = "item_to_read_from_failure"

    @classmethod
    def empty_counter(cls) -> t.Counter["Event"]:
        return Counter({event: 0 for event in cls})


class Reason(str, enum.Enum):
    item_to_read_from_failure = "item_to_read_from_failure"
    origin_does_not_support_incremental = "origin_does_not_support_incremental"
    backend_not_configured_in_incremental_mode = (
        "backend_not_configured_in_incremental_mode"
    )
    workflow_id_not_found = "workflow_id_not_found"
    event_parsing_failure = "event_parsing_failure"
    bad_action_parameters = "bad_action_parameters"
    bad_origin_parameters = "bad_origin_parameters"
    bad_target_parameters = "bad_target_parameters"
    format_failure = "format_failure"
    logics_failure = "logics_failure"
    read_failure = "read_failure"
    write_failure = "write_failure"
    none = ""


class Status(str, enum.Enum):
    success = "success"
    success_with_failures = "success_with_failures"
    fatal = "fatal"


class ActionInitError(BaseModel):
    data: t.Dict
    reason: Reason


class RunResult(BaseModel):
    status: Status
    reason: Reason = Reason.none
    events: t.Counter[Event] = Field(default_factory=Event.empty_counter)
    read_from: t.Optional[str] = None

    @classmethod
    def from_events(cls, events: t.Counter[Event]) -> "RunResult":
        read_success = events[Event.read_success]
        read_failures = events[Event.read_failure]
        if read_success == 0 and read_failures == 0:
            return cls(status=Status.success, events=events)
        elif read_success == 0 and read_failures > 0:
            return cls(
                status=Status.fatal,
                reason=Reason.read_failure,
                events=events,
            )

        format_failures = events[Event.format_failure]
        if format_failures == read_success:
            return cls(
                status=Status.fatal,
                reason=Reason.format_failure,
                events=events,
            )

        logics_failures = events[Event.logics_failure]
        if logics_failures == read_success - format_failures:
            return cls(
                status=Status.fatal,
                reason=Reason.logics_failure,
                events=events,
            )

        logics_discard = events[Event.logics_discard]
        write_failure = events[Event.write_failure]
        if (
            write_failure
            == read_success - format_failures - logics_discard - logics_failures
        ) and write_failure > 0:
            return cls(
                status=Status.fatal,
                reason=Reason.write_failure,
                events=events,
            )

        success_with_failures = any(
            events[event] > 0
            for event in [
                Event.read_failure,
                Event.format_failure,
                Event.logics_failure,
                Event.write_failure,
                Event.callback_failure,
            ]
        )
        if success_with_failures:
            return cls(status=Status.success_with_failures, events=events)
        return cls(status=Status.success, events=events)


LogicFunctionType = t.Callable[[t.Dict], t.Union[t.Dict, None]]
LogicsTemplate = """
import typing as t

def logic_1(item: t.Dict) -> t.Union[t.Dict, None]:
    return None

def logic_2(item: t.Dict) -> t.Uniont[t.Dict, None]:
    return None

logics = [logic_1, logic_2]
"""
LogicsDescription = "List of logic functions"
FormatFunctionType = t.Callable[[t.Dict], t.Dict]
FormatTemplate = """
import typing as t

def format(item: t.Dict) -> t.Dict:
    return item
"""
FormatDescription = "Formatting function"
EventParserFunctionType = t.Callable[[t.Dict], t.Dict]
EventParserTemplate = """
import typing as t

def event_parser(event: t.Dict) -> t.Dict:
    parsed = dict()
    parsed["user_id"] = event["email"]
    parsed["thread_id"] = event["subscription_id"]
    return parsed
"""
EventParserDescription = "Event parsing function"
EventParserExtra = dict(skip_from_docs=True)


class BaseActionParameters(BaseModel):
    logics: t.List[LogicFunctionType] = Field(
        default_factory=list, description=LogicsDescription
    )
    format: FormatFunctionType = Field(lambda x: x, description=FormatDescription)
    event_parser: t.Optional[EventParserFunctionType] = Field(
        None, description=EventParserDescription, **EventParserExtra
    )
    read_mode: ReadMode = Field(
        ReadMode.sync,
        description=(
            "If 'incremental' then `read_from` of the last run is given to Origin"
            " Warehouse during read. **The actual behavior depends on implementation of"
            " read**. In 'sync' mode `read_from` is neither fetched nor given to Origin"
            " Warehouse during read."
        ),
    )

    class Config:
        @staticmethod
        def schema_extra(
            schema: t.Dict[str, t.Any], model: t.Type["BaseActionParameters"]
        ) -> None:
            # JSON has no equivalent for Callable type which is used for
            # logics and format. Thus we hardcode properties for both here
            schema["properties"]["logics"] = {
                "title": "logics",
                "description": (
                    "List of logic functions. Each function should have"
                    " the following signature {}. The final list should be exposed "
                    "in a variable named 'logics'.".format(LogicFunctionType)
                ),
                "template": LogicsTemplate,
                "type": "code_editor",
            }

            schema["properties"]["format"] = {
                "title": "format",
                "description": (
                    "Formatting function. You should expose a function"
                    " named 'format' with following signature {}".format(
                        FormatFunctionType
                    )
                ),
                "template": FormatTemplate,
                "type": "code_editor",
            }

            schema["properties"]["event_parser"] = {
                "title": "event_parser",
                "description": (
                    "Event parsing function for **CATCH** integrations. You should"
                    " expose a function named 'event_parser' with following"
                    " signature {}".format(EventParserFunctionType)
                ),
                "template": EventParserTemplate,
                "type": "code_editor",
            }

    @classmethod
    def with_defaults(
        cls,
        model_name: str,
        *,
        format: t.Optional[FormatFunctionType] = None,
        event_parser: t.Optional[EventParserFunctionType] = None,
    ) -> t.Type["BaseActionParameters"]:
        new_model = cls
        if format is not None:
            new_model = create_model(
                model_name,
                format=(
                    FormatFunctionType,
                    Field(format, description=FormatDescription),
                ),
                __base__=new_model,
            )
        if event_parser is not None:
            new_model = create_model(
                model_name,
                event_parser=(
                    EventParserFunctionType,
                    Field(
                        event_parser,
                        description=EventParserDescription,
                        **EventParserExtra,
                    ),
                ),
                __base__=new_model,
            )
        return new_model


class WorkflowType(str, enum.Enum):
    catch = "hook"
    pull = "schedule"


class ConnectorAction(BaseModel):
    WORKFLOW_FORMAT_PLACEHOLDER = "# << format_placeholder >>"
    WORKFLOW_LOGICS_PLACEHOLDER = "# << logics_placeholder >>"
    WORKFLOW_EVENT_PARSER_PLACEHOLDER = "# << event_parser_placeholder >>"
    ORIGIN_SETTINGS_PREFIX = "origin_"
    TARGET_SETTINGS_PREFIX = "target_"
    WORKFLOW_ID_SETTINGS_KEY = "__workflow_id"

    name: str
    description: str
    trigger_type: WorkflowType
    parameters: t.Type[BaseModel]
    origin: Warehouse
    target: Warehouse
    callback: t.Optional[
        t.Callable[[BaseModel, BaseModel, t.Counter[Event], t.List[t.Dict]], None]
    ] = None

    @validator("origin", pre=False)
    def origin_is_readable(cls, origin):
        if origin.is_readable is False:
            raise ValueError("Origin warehouse is not readable")
        return origin

    @validator("target", pre=False)
    def target_is_writable(cls, target):
        if target.is_writable is False:
            raise ValueError("Target warehouse is not writable")
        return target

    @property
    def data_type(self) -> str:
        return self.origin.data_type.name

    def workflow_code(self, connector_name: str, workflow_type: WorkflowType) -> str:
        return WORKFLOW_TEMPLATE.render(
            format_placeholder=self.WORKFLOW_FORMAT_PLACEHOLDER,
            logics_placeholder=self.WORKFLOW_LOGICS_PLACEHOLDER,
            event_parser_placeholder=self.WORKFLOW_EVENT_PARSER_PLACEHOLDER,
            workflow_id_settings_key=self.WORKFLOW_ID_SETTINGS_KEY,
            origin_settings_prefix=self.ORIGIN_SETTINGS_PREFIX,
            target_settings_prefix=self.TARGET_SETTINGS_PREFIX,
            connector_name=connector_name,
            action_name=self.name,
            type=workflow_type.name,
            origin_parameters=[
                parameter for parameter in self.origin.read.parameters.__fields__
            ],
            target_parameters=[
                parameter for parameter in self.target.write.parameters.__fields__
            ],
        )

    def run(
        self,
        connector_name: str,
        workflow_id: str,
        action_parameters: t.Dict,
        origin_parameters: t.Dict,
        target_parameters: t.Dict,
        init_error: t.Optional[ActionInitError] = None,
    ) -> RunResult:
        action_id = uuid.uuid4()
        adapter = ConnectorActionAdapter(
            logger,
            dict(
                log_tags=[
                    dict(name="connector", value=connector_name),
                    dict(name="action_name", value=self.name),
                    dict(name="workflow_id", value=workflow_id),
                    dict(name="action_id", value=action_id),
                ]
            ),
        )

        if init_error is not None:
            adapter.error(
                "Failed to parse event with reason={} data={}".format(
                    repr(init_error.reason), init_error.data
                )
            )
            return RunResult(
                status=Status.fatal,
                reason=init_error.reason,
            )

        adapter.info("Starting Action")
        try:
            parameters = self.parameters(**action_parameters)
        except ValidationError as e:
            adapter.warning(
                "Failed to parse action_parameters with errors={}".format(e.errors())
            )
            return RunResult(status=Status.fatal, reason=Reason.bad_action_parameters)

        try:
            origin_parameters = self.origin.read.parameters(**origin_parameters)
        except ValidationError as e:
            adapter.warning(
                "Failed to parse origin_parameters with errors={}".format(e.errors())
            )
            return RunResult(status=Status.fatal, reason=Reason.bad_origin_parameters)

        try:
            target_parameters = self.target.write.parameters(**target_parameters)
        except ValidationError as e:
            adapter.warning(
                "Failed to parse target_parameters with errors={}".format(e.errors())
            )
            return RunResult(status=Status.fatal, reason=Reason.bad_target_parameters)

        if parameters.read_mode is ReadMode.incremental:
            if self.origin.supports_incremental is False:
                adapter.warning(
                    "Origin warehouse {} does not support '{}' read mode".format(
                        self.origin.name, ReadMode.incremental.value
                    )
                )
                return RunResult(
                    status=Status.fatal,
                    reason=Reason.origin_does_not_support_incremental,
                )

            if backend.is_configured is False:
                adapter.warning(
                    "For '{}' read_mode backend must be configured".format(
                        ReadMode.incremental.value
                    )
                )
                return RunResult(
                    status=Status.fatal,
                    reason=Reason.backend_not_configured_in_incremental_mode,
                )

        read_from = None
        if parameters.read_mode is ReadMode.incremental:
            adapter.info(
                "Read mode is '{}' fetching last run results".format(
                    ReadMode.incremental.value
                )
            )
            last_results = backend.store.load(key=workflow_id, parse_as=RunResult)
            read_from = last_results.read_from if last_results is not None else None

        events = Event.empty_counter()

        adapter.info(
            "Starting to read from warehouse={} with mode={} read_from={} parameters={}"
            .format(
                self.origin.name,
                parameters.read_mode,
                read_from,
                origin_parameters,
            )
        )
        origin_adapter = ConnectorActionAdapter(
            logger,
            dict(
                log_tags=adapter.extra["log_tags"]
                + [
                    dict(name="warehouse", value=self.origin.name),
                    dict(name="action", value="read"),
                ]
            ),
        )
        origin_items = []
        try:
            for item in self.origin.read(
                origin_adapter,
                origin_parameters,
                read_mode=parameters.read_mode,
                read_from=read_from,
            ):
                origin_items.append(item)
                events[Event.read_success] += 1
        except Exception as e:
            events[Event.read_failure] += 1
            adapter.error(
                "Failed to read from warehouse={} with parameters={} error={}".format(
                    self.origin.name, origin_parameters, repr(e)
                )
            )
        adapter.info(
            "Finished reading from warehouse={} n_items={} read_failure={}".format(
                self.origin.name,
                len(origin_items),
                events[Event.read_failure] > 0,
            )
        )

        next_read_from = read_from
        if len(origin_items) > 0 and parameters.read_mode is ReadMode.incremental:
            last_item = origin_items[-1]
            try:
                next_read_from = self.origin.item_to_read_from(last_item)
            except Exception as e:
                events[Event.item_to_read_from_failure] += 1
                adapter.error(
                    "Failed to get read_from from warehouse={} with parameters={}"
                    " item={} error={}".format(
                        self.origin.name, origin_parameters, last_item, repr(e)
                    )
                )
                return RunResult(
                    status=Status.fatal,
                    reason=Reason.item_to_read_from_failure,
                    events=events,
                )

        using_default_format = not bool(action_parameters.get("format"))
        adapter.info(
            "Starting to format origin items using {} function".format(
                "default" if using_default_format else "user defined"
            )
        )
        formatted_items = []
        for item in origin_items:
            try:
                formatted_items.append(parameters.format(item))
            except Exception as e:
                events[Event.format_failure] += 1
                adapter.error(
                    "Failed to format origin item using {} function error={}".format(
                        "default" if using_default_format else "user defined", repr(e)
                    )
                )
        adapter.info(
            "Finished formatting origin items success={} failures={}".format(
                len(formatted_items), events[Event.format_failure]
            )
        )

        if len(parameters.logics) > 0:
            adapter.info(
                "Starting to apply logic functions: "
                "n_items={} before applying logics".format(len(formatted_items))
            )
            items_to_write = []
            for item in formatted_items:
                for i, logic in enumerate(parameters.logics):
                    try:
                        item = logic(item)
                    except Exception as e:
                        adapter.error(
                            "Failed to apply logic function number={} error={}".format(
                                i, repr(e)
                            )
                        )
                        events[Event.logics_failure] += 1
                        break
                    if item is None:
                        events[Event.logics_discard] += 1
                        break
                else:
                    items_to_write.append(item)
            adapter.info(
                "Finished applying logic functions: "
                "success={} discarded={} failures={}".format(
                    len(items_to_write),
                    events[Event.logics_discard],
                    events[Event.logics_failure],
                )
            )
        else:
            adapter.info("No logic functions supplied. Skipping")
            items_to_write = formatted_items

        adapter.info(
            "Starting to write to warehouse={} with parameters={} n_items={}".format(
                self.target.name, target_parameters, len(items_to_write)
            )
        )
        target_adapter = ConnectorActionAdapter(
            logger,
            dict(
                log_tags=adapter.extra["log_tags"]
                + [
                    dict(name="warehouse", value=self.target.name),
                    dict(name="action", value="write"),
                ]
            ),
        )
        try:
            failed_items = self.target.write(
                target_adapter, target_parameters, items_to_write
            )
            events[Event.write_failure] += len(failed_items)
        except Exception as e:
            adapter.error(
                "Failed to write to warehouse={} with parameters={} error={}".format(
                    self.target.name, target_parameters, repr(e)
                )
            )
            events[Event.write_failure] += len(items_to_write)
            return RunResult(
                status=Status.fatal,
                reason=Reason.write_failure,
                events=events,
            )
        adapter.info(
            "Finished writing to warehouse={} success={} failures={}".format(
                self.target.name,
                len(items_to_write) - events[Event.write_failure],
                events[Event.write_failure],
            )
        )

        if self.callback is not None:
            adapter.info("Calling callback function")
            try:
                self.callback(
                    origin_parameters, target_parameters, events, items_to_write
                )
            except Exception as e:
                events[Event.callback_failure] += 1
                adapter.error("Failed to run callback with error={}".format(repr(e)))
            finally:
                events[Event.callback_executed] += 1

        results = RunResult.from_events(events)
        results.read_from = next_read_from
        if backend.is_configured:
            adapter.info("Saving run results in {} backend".format(backend.store.name))
            backend.store.save(key=workflow_id, data=results)

        adapter.info("Finished action")
        return results


class ConnectorModel(BaseModel):
    name: str
    description: str
    url: str
    actions: t.List[ConnectorAction]

    def action_by_name(self, action_name: str) -> t.Optional[ConnectorAction]:
        if "__actions_by_name" not in self.__dict__:
            self.__dict__["__actions_by_name"] = {
                action.name: action for action in self.actions
            }
        return self.__dict__["__actions_by_name"].get(action_name)


class Connector:
    def __init__(self, *args, **kwargs) -> None:
        self.model = ConnectorModel(*args, **kwargs)
        for action in self.model.actions:
            with_connector_name = partial(action.run, connector_name=self.model.name)
            setattr(self, action.name, with_connector_name)

    def manifest(self) -> t.Dict:
        model = self.model
        manifest = dict(name=model.name, actions=[])
        for action in model.actions:
            format_placeholder = action.WORKFLOW_FORMAT_PLACEHOLDER
            logics_placeholder = action.WORKFLOW_LOGICS_PLACEHOLDER
            event_parser_placeholder = action.WORKFLOW_EVENT_PARSER_PLACEHOLDER
            action_manifest = dict(
                name=action.name,
                action_parameters=copy.deepcopy(action.parameters.schema()),
                data_type=action.data_type,
                trigger_type=action.trigger_type.value,
                origin=action.origin.name,
                origin_parameters=action.origin.read.parameters.schema(),
                origin_data_schema=action.origin.data_schema.schema(),
                supports_incremental=action.origin.supports_incremental,
                target=action.target.name,
                target_parameters=action.target.write.parameters.schema(),
                target_data_schema=action.target.data_schema.schema(),
                workflow_code=action.workflow_code(
                    connector_name=model.name, workflow_type=action.trigger_type
                ),
                workflow_code_format_placeholder=format_placeholder,
                workflow_code_logics_placeholder=logics_placeholder,
                workflow_code_event_parser_placeholder=event_parser_placeholder,
                workflow_code_workflow_id_settings_key=action.WORKFLOW_ID_SETTINGS_KEY,
                workflow_code_origin_settings_prefix=action.ORIGIN_SETTINGS_PREFIX,
                workflow_code_target_settings_prefix=action.TARGET_SETTINGS_PREFIX,
            )
            if action.trigger_type is WorkflowType.pull:
                action_manifest.pop("workflow_code_event_parser_placeholder")
                action_manifest["action_parameters"]["properties"].pop("event_parser")

            manifest["actions"].append(action_manifest)
        return manifest


def hrflow_connectors_manifest(
    connectors: t.List[Connector], directory_path: str = "."
) -> None:
    manifest = dict(
        name="HrFlow.ai Connectors",
        connectors=[connector.manifest() for connector in connectors],
    )
    with open("{}/manifest.json".format(directory_path), "w") as f:
        json.dump(manifest, f, indent=2)



from hrflow_connectors.connectors.adzuna.connector import Adzuna
from hrflow_connectors.connectors.breezyhr import BreezyHR
from hrflow_connectors.connectors.bullhorn import Bullhorn
from hrflow_connectors.connectors.ceridian import Ceridian
from hrflow_connectors.connectors.greenhouse.connector import Greenhouse
from hrflow_connectors.connectors.hubspot import Hubspot
from hrflow_connectors.connectors.poleemploi import PoleEmploi
from hrflow_connectors.connectors.recruitee import Recruitee
from hrflow_connectors.connectors.sapsuccessfactors import SAPSuccessFactors
from hrflow_connectors.connectors.smartrecruiters import SmartRecruiters
from hrflow_connectors.connectors.talentsoft import TalentSoft
from hrflow_connectors.connectors.teamtailor import Teamtailor
from hrflow_connectors.connectors.waalaxy import Waalaxy
from hrflow_connectors.connectors.workable import Workable
from hrflow_connectors.core import backend
from hrflow_connectors.core.connector import hrflow_connectors_manifest  # noqa
from hrflow_connectors.core.documentation import generate_docs  # noqa

__version__ = "2.0.0"
__CONNECTORS__ = [
    SmartRecruiters,
    TalentSoft,
    PoleEmploi,
    Adzuna,
    Recruitee,
    Workable,
    BreezyHR,
    SAPSuccessFactors,
    Bullhorn,
    Ceridian,
    Greenhouse,
    Teamtailor,
    Waalaxy,
    Hubspot,
]

# This makes sure that connector are in module namespace
# and that the automatic workflow code generation should work
for connector in __CONNECTORS__:
    globals()[connector.model.name] = connector

backend.configure_store()



import json
import re

from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.workable.warehouse import (
    WorkableJobWarehouse,
    WorkableProfileWarehouse,
)
from hrflow_connectors.core.connector import (
    BaseActionParameters,
    Connector,
    ConnectorAction,
    WorkflowType,
)

from ..hrflow.schemas import HrFlowJob, HrFlowProfile
from .schemas import WorkableCandidate, WorkableJobModel


def remove_html_tags(text: str) -> str:
    """
    Remove all HTML tags in a string
    Args:
        text (str): text to clean
    Returns:
        str: cleaned text (without HTML tags)
    """
    return re.sub("<[^<]+?>", "", text)


def format_jobs(workable_job: WorkableJobModel) -> HrFlowJob:
    """
    Format a job into the hrflow job object format
    Args:
        data (WorkableJobModel): a job object pulled from workable subdomain
    Returns:
        HrflowJob: a job into the hrflow job object format
    """
    hrflow_job = dict()
    # name and reference
    hrflow_job["name"] = workable_job.get("title")
    hrflow_job["reference"] = workable_job.get("shortcode")
    # url
    hrflow_job["url"] = workable_job.get("url")
    # location
    location = workable_job.get("location")
    location_str = location.get("location_str")
    text = None
    if isinstance(location_str, str):
        text = location_str
    geojson = dict()

    def get_geojson(field_name: str):
        if location.get(field_name) is not None:
            geojson["field_name"] = location.get(field_name)

    get_geojson("country")
    get_geojson("country_code")
    get_geojson("region_code")
    get_geojson("region")
    get_geojson("city")
    get_geojson("zip_code")
    get_geojson("telecommuting")
    hrflow_job["location"] = dict(text=text, geojson=geojson)
    # sections
    hrflow_job["sections"] = []

    def create_section(field_name: str):
        title_name = f"workable_{field_name}"
        field_value = workable_job.get(field_name)
        if isinstance(field_value, str):
            description = remove_html_tags(field_value)
            section = dict(name=title_name, title=title_name, description=description)
            hrflow_job["sections"].append(section)

    create_section("description")
    create_section("requirements")
    create_section("benefits")
    # creation_date
    hrflow_job["created_at"] = workable_job.get("created_at")
    # tags
    hrflow_job["tags"] = []

    def create_tag(field_name):
        name = f"workable_{field_name}"
        field_value = workable_job.get("field_name")
        if field_value is not None:
            tag = dict(name=name, value=field_value)
            hrflow_job["tags"].append(tag)

    create_tag("employment_type")
    create_tag("full_title")
    create_tag("id")
    create_tag("code")
    create_tag("state")
    create_tag("department")
    create_tag("application_url")
    create_tag("shortlink")
    create_tag("employment_type")

    return hrflow_job


def format_profile(
    hrflow_profile: HrFlowProfile,
) -> WorkableCandidate:
    """
    Format a HrflowProfile object into a WorkableCandidate object
    Args:
        data (HrflowProfile): HrflowProfile object
    Returns:
        WorkableCandidate: WorkableCandidate object
    """
    candidate_profile = dict()

    info = hrflow_profile.get("info")

    candidate_profile["name"] = info.get("full_name")
    candidate_profile["summary"] = info.get("summary")
    candidate_profile["email"] = info.get("email")
    candidate_profile["phone"] = info.get("phone")
    location = info.get("location")

    if isinstance(location.get("text"), str):
        candidate_profile["address"] = location.get("text")
    attachments = hrflow_profile.get("attachments")

    if isinstance(attachments, list):
        for attachment in attachments:
            if isinstance(attachment, dict):
                if attachment["type"] == "resume":
                    candidate_profile["resume_url"] = attachment["public_url"]

    workable_profile = dict(sourced=True, candidate=candidate_profile)
    return json.dumps(workable_profile)


Workable = Connector(
    name="Workable",
    description=(
        "More than an applicant tracking system, "
        "Workable's talent acquisition software helps teams find candidates, "
        "evaluate applicants and make the right hire, faster."
    ),
    url="https://www.workable.com/",
    actions=[
        ConnectorAction(
            name="pull_jobs",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs via the ***Workable*** API and send them"
                " to a ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "PullJobsActionParameters", format=format_jobs
            ),
            origin=WorkableJobWarehouse,
            target=HrFlowJobWarehouse,
        ),
        ConnectorAction(
            name="push_profile",
            trigger_type=WorkflowType.catch,
            description=(
                "Writes a profile from ***Hrflow.ai Source*** to ***Workable*** via the"
                " API for the given `shortcode`."
            ),
            parameters=BaseActionParameters.with_defaults(
                "WriteProfileActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=WorkableProfileWarehouse,
        ),
    ],
)



import typing as t
from logging import LoggerAdapter

from pydantic import Field

from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
)


class ReadProfilesParameters(ParametersModel):
    profile: t.Dict = Field(
        None,
        description="Profile object recieved from the Webhook",
        field_type=FieldType.Other,
    )


def read(
    adapter: LoggerAdapter,
    parameters: ReadProfilesParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    return [parameters.profile]


WaalaxyProfilesWarehouse = Warehouse(
    name="Waalaxy Profiles",
    data_type=DataType.profile,
    read=WarehouseReadAction(
        parameters=ReadProfilesParameters,
        function=read,
    ),
)



import json
import os
import typing as t
from contextlib import contextmanager
from contextvars import ContextVar
from importlib import import_module
from pathlib import Path

import yaml
from pydantic import BaseModel, Field, StrictStr, ValidationError

from hrflow_connectors.core.connector import Connector, Event, Reason, Status
from hrflow_connectors.core.warehouse import ReadMode, Warehouse

PROJECT_DIRECTORY = Path(__file__).parent.parent.parent.parent
CONNECTORS_DIRECTORY = Path(__file__).parent.parent / "connectors"

SECRETS_PREFIX = "$__"
ENVIRON_SECRETS_PREFIX = "HRFLOW_CONNECTORS_{connector_name}_"
Secrets = ContextVar("secrets")
ConnectorName = ContextVar("connector_name")
ActionNames = ContextVar("action_names")
WarehouseNames = ContextVar("warehouse_names")


class NoTestConfigException(Exception):
    pass


class InvalidJSONException(Exception):
    pass


class InvalidYAMLException(Exception):
    pass


class InvalidTestConfigException(Exception):
    pass


@contextmanager
def secrets(connector_name: str, connectors_directory: Path):
    secrets_prefix = ENVIRON_SECRETS_PREFIX.format(
        connector_name=connector_name.upper()
    )

    global_secrets_file = connectors_directory / "secrets.json"
    if global_secrets_file.exists():
        try:
            global_secrets = json.loads(global_secrets_file.read_text())
            global_secrets = {
                key.replace(secrets_prefix, "", 1): value
                for key, value in global_secrets.items()
                if key.startswith(secrets_prefix)
            }
        except json.JSONDecodeError as e:
            raise InvalidJSONException(
                "Failed to JSON decode global secrets file for connector {} with"
                " error {}".format(connector_name, e)
            )
    else:
        global_secrets = dict()

    connector_secrets_file = (
        connectors_directory / connector_name.lower() / "secrets.json"
    )
    if connector_secrets_file.exists():
        try:
            connector_secrets = json.loads(connector_secrets_file.read_text())
        except json.JSONDecodeError as e:
            raise InvalidJSONException(
                "Failed to JSON decode secrets file for connector {} "
                "with error {}".format(connector_name, e)
            )
    else:
        connector_secrets = dict()

    environnment_secrets = {
        key.replace(secrets_prefix, "", 1): value
        for key, value in os.environ.items()
        if key.startswith(secrets_prefix)
    }
    token = Secrets.set({**global_secrets, **connector_secrets, **environnment_secrets})
    yield
    Secrets.reset(token)


@contextmanager
def actions(connector: Connector):
    connector_name_token = ConnectorName.set(connector.model.name)
    actions_token = ActionNames.set([action.name for action in connector.model.actions])
    yield
    ConnectorName.reset(connector_name_token)
    ActionNames.reset(actions_token)


@contextmanager
def warehouses(connector_name: str, connectors_directory: Path):
    if connectors_directory is CONNECTORS_DIRECTORY:  # pragma: no cover
        warehouse_module = import_module(
            "hrflow_connectors.connectors.{}.warehouse".format(connector_name.lower())
        )
    else:
        import_from = connectors_directory.relative_to(PROJECT_DIRECTORY)
        warehouse_module = import_module(
            "{}.{}.warehouse".format(
                str(import_from).replace("/", "."), connector_name.lower()
            )
        )
    warehouse_names = []
    for attribute, value in warehouse_module.__dict__.items():
        if isinstance(value, Warehouse):
            warehouse_names.append(attribute)
    token = WarehouseNames.set(warehouse_names)
    yield
    WarehouseNames.reset(token)


class ParameterValue(object):
    @classmethod
    def validate(cls, v):
        if isinstance(v, str) and v.startswith(SECRETS_PREFIX):
            key = v.replace(SECRETS_PREFIX, "", 1)
            if key not in Secrets.get(dict()):
                raise TypeError(
                    "{} not found in secrets for connector {}".format(
                        key, ConnectorName.get("")
                    )
                )
            return Secrets.get()[key]
        return v

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def __modify_schema__(cls, field_schema):
        field_schema.update(
            description=(
                "If the value is a string starting with {} then it's fetched from"
                " environnment and secrets.json file".format(SECRETS_PREFIX)
            ),
            examples=["$API_KEY"],
        )


class ActionName(StrictStr):
    @classmethod
    def validate(cls, v):
        action_names = ActionNames.get([])
        if v not in action_names:
            raise TypeError(
                "No action '{}' found for connector {}. Should be one of {}".format(
                    v, ConnectorName.get(""), action_names
                )
            )
        return v

    @classmethod
    def __get_validators__(cls):
        for validator in super().__get_validators__():
            yield validator
        yield cls.validate


class WarehouseName(StrictStr):
    @classmethod
    def validate(cls, v):
        warehoue_names = WarehouseNames.get([])
        if v not in warehoue_names:
            raise TypeError(
                "Warehouse '{}' not found for connector {}. Should be one of {}".format(
                    v, ConnectorName.get(""), warehoue_names
                )
            )
        return v

    @classmethod
    def __get_validators__(cls):
        for validator in super().__get_validators__():
            yield validator
        yield cls.validate


class ReadTest(BaseModel):
    id: t.Optional[str] = None
    parameters: t.Dict[str, ParameterValue]
    read_mode: t.Optional[ReadMode] = None
    read_from: t.Optional[str] = None
    expected_number_of_items: t.Optional[int] = None


class WarehouseTests(BaseModel):
    read: t.List[ReadTest] = Field(default_factory=list)


class ActionTest(BaseModel):
    id: t.Optional[str] = None
    origin_parameters: t.Dict[str, ParameterValue]
    target_parameters: t.Dict[str, ParameterValue]
    status: t.Optional[Status]
    reason: t.Optional[Reason]
    events: t.Optional[t.Counter[Event]]


class ConnectorTestConfig(BaseModel):
    warehouse: t.Dict[WarehouseName, WarehouseTests] = Field(default_factory=dict)
    actions: t.Dict[ActionName, t.List[ActionTest]] = Field(default_factory=dict)


def collect_connector_tests(
    connector: Connector, connectors_directory: Path = CONNECTORS_DIRECTORY
):
    connector_name = connector.model.name
    test_config_file = (
        connectors_directory / connector_name.lower() / "test-config.yaml"
    )
    if test_config_file.exists() is False:
        raise NoTestConfigException(
            "No test configuration found for connector {} at {}".format(
                connector_name, test_config_file
            )
        )
    try:
        test_config = yaml.safe_load(test_config_file.read_bytes())
    except yaml.YAMLError as e:
        raise InvalidYAMLException(
            "Failed to parse yaml test config for connector {} with error {} ".format(
                connector_name, str(e)
            )
        )

    try:
        with warehouses(
            connector_name=connector_name,
            connectors_directory=connectors_directory,
        ):
            with actions(
                connector=connector,
            ):
                with secrets(
                    connector_name=connector_name,
                    connectors_directory=connectors_directory,
                ):
                    test_suite = ConnectorTestConfig(**test_config)
    except ValidationError as e:
        raise InvalidTestConfigException(e.errors())

    return test_suite



import typing as t
from logging import LoggerAdapter

from hrflow import Hrflow
from pydantic import Field

from hrflow_connectors.connectors.hrflow.schemas import (
    HrFlowProfile,
    HrFlowProfileParsing,
)
from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
    WarehouseWriteAction,
)


class ReadProfileParameters(ParametersModel):
    api_secret: str = Field(
        ...,
        description="X-API-KEY used to access HrFlow.ai API",
        repr=False,
        field_type=FieldType.Auth,
    )
    api_user: str = Field(
        ...,
        description="X-USER-EMAIL used to access HrFlow.ai API",
        field_type=FieldType.Auth,
    )
    source_key: str = Field(
        ..., description="HrFlow.ai source key", field_type=FieldType.QueryParam
    )
    profile_key: str = Field(
        ..., description="HrFlow.ai profile key", field_type=FieldType.QueryParam
    )


class WriteProfileParameters(ParametersModel):
    api_secret: str = Field(
        ...,
        description="X-API-KEY used to access HrFlow.ai API",
        repr=False,
        field_type=FieldType.Auth,
    )
    api_user: str = Field(
        ...,
        description="X-USER-EMAIL used to access HrFlow.ai API",
        field_type=FieldType.Auth,
    )
    source_key: str = Field(
        ..., description="HrFlow.ai source key", field_type=FieldType.QueryParam
    )
    edit: bool = Field(
        False,
        description="When enabled the profile must exist in the source",
        field_type=FieldType.Other,
    )
    only_edit_fields: t.Optional[t.List[str]] = Field(
        ...,
        description=(
            "List of attributes to use for the edit operation e.g. ['tags',"
            " 'metadatas']"
        ),
        field_type=FieldType.Other,
    )


class WriteProfileParsingParameters(ParametersModel):
    api_secret: str = Field(
        ...,
        description="X-API-KEY used to access HrFlow.ai API",
        repr=False,
        field_type=FieldType.Auth,
    )
    api_user: str = Field(
        ...,
        description="X-USER-EMAIL used to access HrFlow.ai API",
        field_type=FieldType.Auth,
    )
    source_key: str = Field(
        ..., description="HrFlow.ai source key", field_type=FieldType.Other
    )
    only_insert: bool = Field(
        False,
        description=(
            "When enabled the profile is written only if it doesn't exist in the source"
        ),
        field_type=FieldType.Other,
    )


def read(
    adapter: LoggerAdapter,
    parameters: ReadProfileParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.List[t.Dict]:
    hrflow_client = Hrflow(
        api_secret=parameters.api_secret, api_user=parameters.api_user
    )
    response = hrflow_client.profile.indexing.get(
        source_key=parameters.source_key, key=parameters.profile_key
    )
    if "Unable to find object" in response["message"]:
        adapter.info(
            "No profile found for source_key={} profile_key={} response={}".format(
                parameters.source_key, parameters.profile_key, response
            )
        )
        return []
    elif response["code"] >= 400:
        adapter.error(
            "Failed to get profile source_key={} profile_key={} response={}".format(
                parameters.source_key, parameters.profile_key, response
            )
        )
        raise Exception("Failed to get profile")
    return [response["data"]]


def write(
    adapter: LoggerAdapter,
    parameters: WriteProfileParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    failed = []
    hrflow_client = Hrflow(
        api_secret=parameters.api_secret, api_user=parameters.api_user
    )
    for profile in profiles:
        if parameters.edit:
            current_profile = hrflow_client.profile.indexing.get(
                source_key=parameters.source_key, reference=profile["reference"]
            ).get("data")
            if not current_profile:
                adapter.warning(
                    "Mode edit is activated and profile with reference={} not found"
                    " in source. Failing for profile...".format(profile["reference"])
                )
                failed.append(profile)
                continue
            if parameters.only_edit_fields:
                edit = {
                    field: profile.get(field) for field in parameters.only_edit_fields
                }
            else:
                edit = profile
            profile_to_index = {**current_profile, **edit}

            response = hrflow_client.profile.indexing.edit(
                source_key=parameters.source_key,
                key=current_profile["key"],
                profile_json=profile_to_index,
            )
            if response["code"] != 200:
                adapter.error(
                    "Failed to edit profile with reference={} key={} response={}"
                    .format(
                        profile_to_index["key"], profile_to_index["reference"], response
                    )
                )
                failed.append(profile)
        else:
            response = hrflow_client.profile.indexing.add_json(
                source_key=parameters.source_key, profile_json=profile
            )
            if response["code"] // 100 != 2:
                adapter.error("Failed to add profile with response={}".format(response))
                failed.append(profile)
    return failed


def write_parsing(
    adapter: LoggerAdapter,
    parameters: WriteProfileParsingParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    failed = []
    hrflow_client = Hrflow(
        api_secret=parameters.api_secret, api_user=parameters.api_user
    )
    for profile in profiles:
        if parameters.only_insert and hrflow_client.profile.indexing.get(
            source_key=parameters.source_key, reference=profile["reference"]
        ).get("data"):
            adapter.info(
                "Mode only_insert is activated. Profile with reference={} already"
                " in source. Skipping...".format(profile["reference"])
            )
            continue

        response = hrflow_client.profile.parsing.add_file(
            source_key=parameters.source_key,
            profile_file=profile["resume"]["raw"],
            profile_content_type=profile["resume"]["content_type"],
            reference=profile["reference"],
            tags=profile["tags"],
            metadatas=profile["metadatas"],
            created_at=profile["created_at"],
        )
        if response["code"] != 202:
            adapter.error(
                "Failed to parse profile with reference={} response={}".format(
                    profile["reference"], response
                )
            )
            failed.append(profile)
    return failed


HrFlowProfileWarehouse = Warehouse(
    name="HrFlow.ai Profiles",
    data_schema=HrFlowProfile,
    data_type=DataType.profile,
    read=WarehouseReadAction(parameters=ReadProfileParameters, function=read),
    write=WarehouseReadAction(parameters=WriteProfileParameters, function=write),
)


HrFlowProfileParsingWarehouse = Warehouse(
    name="HrFlow.ai Profile Parsing",
    data_schema=HrFlowProfileParsing,
    data_type=DataType.profile,
    write=WarehouseWriteAction(
        parameters=WriteProfileParsingParameters, function=write_parsing
    ),
)



import typing as t

from hrflow_connectors.connectors.ceridian.warehouse import CeridianJobWarehouse
from hrflow_connectors.connectors.hrflow.warehouse import HrFlowJobWarehouse
from hrflow_connectors.core import (
    BaseActionParameters,
    Connector,
    ConnectorAction,
    WorkflowType,
)

from .schemas import CeridianDayforceJobModel


def format_job(data: CeridianDayforceJobModel) -> t.Dict:
    """
    format a job into the hrflow job object format
    Args:
        data (CeridianDayforceJobModel): a job object pulled from a ceridian
        dayforce job feed space
    Returns:
        HrflowJob: a job into the hrflow job object format
    """
    job = dict()

    # basic information
    job["name"] = data.get("Title")
    job["summary"] = None
    job["reference"] = str(data.get("ReferenceNumber")) + str(
        data.get("ParentRequisitionCode")
    )
    job["url"] = data.get("JobDetailsUrl")

    # location
    location = data.get("City")
    state = data.get("State")
    country = data.get("Country")
    postal_code = data.get("PostalCode")
    geojson = dict(state=state, country=country, postal_code=postal_code)
    job["location"] = dict(text=location, lat=None, lng=None, geojson=geojson)

    # sections
    description = data.get("Description")
    job["sections"] = [
        dict(
            name="dayforce_description",
            title="dayforce_description",
            description=description,
        )
    ]
    job["created_at"] = data.get("DatePosted")
    job["updated_at"] = data.get("LastUpdated")

    # tags
    apply_url = str(data.get("ApplyUrl"))
    client_site_name = str(data.get("ClientSiteName"))
    client_site_ref_code = str(data.get("ClientSiteXRefCode"))
    company_name = str(data.get("CompanyName"))
    remote = str(data.get("IsVirtualLocation"))
    job["tags"] = [
        dict(name="dayforce_apply_url", value=apply_url),
        dict(name="dayforce_client-site-name", value=client_site_name),
        dict(name="dayforce_client-site-ref-code", value=client_site_ref_code),
        dict(name="dayforce_company_name", value=company_name),
        dict(name="dayforce_remote", value=remote),
    ]

    return job


DESCRIPTION = (
    "Dayforce enterprise HCM software combines payroll, HR, benefits, talent and"
    " workforce management in a single cloud application to power the future of"
    " work.Ceridian"
)

Ceridian = Connector(
    name="Ceridian",
    description=DESCRIPTION,
    url="https://www.ceridian.com/",
    actions=[
        ConnectorAction(
            name="pull_jobs",
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieve all jobs via the ***Ceridian*** API and send them"
                " to an ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadJobsActionParameters", format=format_job
            ),
            origin=CeridianJobWarehouse,
            target=HrFlowJobWarehouse,
        ),
    ],
)
